@@ -8,53 +8,31 @@ How to find and plot the location of an active region on an HMI magnetogram.
 import matplotlib.pyplot as plt
 import numpy as np
 
-import astropy.units as u
 from astropy.coordinates import SkyCoord
-from astropy.time import TimeDelta
 
 import sunpy.coordinates
 import sunpy.data.sample
 import sunpy.map
 from sunpy.io.special import srs
-from sunpy.net import Fido
-from sunpy.net import attrs as a
-from sunpy.time import parse_time
 
 ##############################################################################
-# For this example, we will start with the sample data. We need an HMI
-# file and use it to create a map.
+# For this example, we will start with the sample data. We need an HMI file and
+# use it to create a map, and the SRS table which contains a list of active
+# regions. Both of these data can be downloaded with ``Fido``.
 
 smap = sunpy.map.Map(sunpy.data.sample.HMI_LOS_IMAGE)
-
-##############################################################################
-# Then using the observation time of the sample image we will download the corresponding
-# NOAA SWPC solar region summary. These are published once a day, so we will define an
-# ``end_time`` that ends just before the next day, otherwise you will download two files.
-
-start_time = parse_time(smap.date)
-end_time = start_time + TimeDelta(23*u.hour + 59*u.minute + 59*u.second)
-
-##############################################################################
-#  Here we use `sunpy.net.Fido` to acquire the data.
-
-srs_results = Fido.search(a.Time(start_time, end_time), a.Instrument.srs_table)
-srs_downloaded_files = Fido.fetch(srs_results)
-
-##############################################################################
-# To read this file, we pass the filename into the SRS reader. So now
-# `srs_table` contains an astropy table.
-srs_table = srs.read_srs(srs_downloaded_files[0])
-print(srs_table)
+srs_table = srs.read_srs(sunpy.data.sample.SRS_TABLE)
 
 ##############################################################################
 # We only need the rows which have 'ID' = 'I' or 'IA'.
+# Some tables do not have these columns, so we exit the script if they are not
+# present.
 
 if 'I' in srs_table['ID'] or 'IA' in srs_table['ID']:
     srs_table = srs_table[np.logical_or(srs_table['ID'] == 'I',
                                         srs_table['ID'] == 'IA')]
 else:
-    print("Warning : No I or IA entries for this date.")
-    srs_table = None
+    raise ValueError("No I or IA entries for this date.")
 
 ##############################################################################
 # Now we extract the latitudes, longitudes and the region numbers. We make an
@@ -81,5 +59,8 @@ if len(lats) > 0:
 
     for i, num in enumerate(numbers):
         ax.annotate(num, (lngs[i].value, lats[i].value),
-                    xycoords=ax.get_transform('heliographic_stonyhurst'))
+                    xycoords=ax.get_transform('heliographic_stonyhurst'),
+                    color='red',
+                    fontweight='bold',
+                    )
 plt.show()
