@@ -20,7 +20,7 @@ from sunpy.net import attrs as a
 # The first step is to download some data, we are going to get an image from
 # early 2011 when the STEREO spacecraft were roughly 90 deg seperated from the
 # Earth.
-stereo = (a.vso.Source('STEREO_B') &
+stereo = (a.Source('STEREO_B') &
           a.Instrument("EUVI") &
           a.Time('2011-01-01', '2011-01-01T00:10:00'))
 
@@ -78,8 +78,8 @@ subaia.plot()
 
 ###############################################################################
 # We now want to crop out this same area on the STEREO EUVI image. First, we
-# create a `SkyCoord` object with the four corners of the box. When we create
-# this object, we use `Map.coordinate_frame` so that the location parameters of
+# create a `~astropy.coordinates.SkyCoord` object with the four corners of the box. When we create
+# this object, we use `sunpy.map.GenericMap.coordinate_frame` so that the location parameters of
 # SDO are correctly set.
 corners = ([aia_bottom_left.Tx, aia_bottom_left.Ty],
            [aia_top_right.Tx, aia_bottom_left.Ty],
