@@ -11,6 +11,7 @@ from astropy.coordinates import (
     Angle,
     Distance,
     GeocentricMeanEcliptic,
+    GeocentricTrueEcliptic,
     HeliocentricMeanEcliptic,
     Latitude,
     Longitude,
@@ -120,7 +121,9 @@ def carrington_rotation_time(crot):
     # Perform two iterations of the correction to achieve sub-second accuracy
     estimate = refine(estimate)
     estimate = refine(estimate)
-    return Time(estimate, scale='tt', format='jd')
+    t = Time(estimate, scale='tt', format='jd').utc
+    t.format = 'iso'
+    return t
 
 
 @add_common_docstring(**_variables_for_parse_time_docstring())
@@ -196,12 +199,10 @@ def apparent_longitude(t='now'):
     """
     time = parse_time(t)
     sun = SkyCoord(0*u.deg, 0*u.deg, 0*u.AU, frame='hcrs', obstime=time)
-    coord = sun.transform_to(GeocentricMeanEcliptic(equinox=time))
+    coord = sun.transform_to(GeocentricTrueEcliptic(equinox=time))
 
-    # Astropy's GeocentricMeanEcliptic already includes aberration, so only add nutation
-    jd1, jd2 = get_jd12(time, 'tt')
-    nut_lon, _ = erfa.nut06a(jd1, jd2)*u.radian
-    lon = coord.lon + nut_lon
+    # Astropy's GeocentricTrueEcliptic includes both aberration and nutation
+    lon = coord.lon
 
     return Longitude(lon)
 
@@ -241,9 +242,9 @@ def apparent_latitude(t='now'):
     """
     time = parse_time(t)
     sun = SkyCoord(0*u.deg, 0*u.deg, 0*u.AU, frame='hcrs', obstime=time)
-    coord = sun.transform_to(GeocentricMeanEcliptic(equinox=time))
+    coord = sun.transform_to(GeocentricTrueEcliptic(equinox=time))
 
-    # Astropy's GeocentricMeanEcliptic does not include nutation, but the contribution is negligible
+    # Astropy's GeocentricTrueEcliptic includes both aberration and nutation
     lat = coord.lat
 
     return Latitude(lat)
@@ -471,10 +472,10 @@ def B0(time='now'):
 
     Returns
     -------
-    out : `~astropy.coordinates.Angle`
+    out : `~astropy.coordinates.Latitude`
         The position angle
     """
-    return Angle(get_earth(time).lat)
+    return Latitude(get_earth(time).lat)
 
 
 # Function returns a SkyCoord's longitude in the de-tilted frame (HCRS rotated so that the Sun's
