@@ -7,7 +7,7 @@ import pytest
 import sunpy.data.test
 import sunpy.map
 from sunpy.instr import iris
-from sunpy.util import SunpyUserWarning
+from sunpy.util.exceptions import SunpyMetadataWarning
 
 
 def test_SJI_to_sequence():
@@ -26,7 +26,7 @@ def test_iris_rot():
                              'iris_l2_20130801_074720_4040000014_SJI_1400_t000.fits')
     iris_cube = iris.SJI_to_sequence(test_data, start=0, stop=None, hdu=0)
     irismap = iris_cube.maps[0]
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
+    with pytest.warns(SunpyMetadataWarning, match='Missing metadata for observer'):
         irismap_rot = irismap.rotate()
 
     assert isinstance(irismap_rot, sunpy.map.sources.SJIMap)
