@@ -26,6 +26,7 @@ Notes
 import os
 import re
 import sys
+import math
 import warnings
 import traceback
 import collections
@@ -138,7 +139,9 @@ def get_header(afile):
                 if card.comment != '':
                     keydict.update({card.keyword: card.comment})
             header['KEYCOMMENTS'] = keydict
-            header['WAVEUNIT'] = extract_waveunit(header)
+            waveunit = extract_waveunit(header)
+            if waveunit is not None:
+                header['WAVEUNIT'] = waveunit
 
             headers.append(header)
     finally:
@@ -216,6 +219,11 @@ def header_to_fits(header):
                           SunpyUserWarning)
             continue
 
+        if isinstance(v, float) and math.isnan(v):
+            warnings.warn(f'The meta key {k} has a NaN value, which is not valid in a FITS '
+                          'header, dropping from the FITS header', SunpyUserWarning)
+            continue
+
         if k.upper() in ('COMMENT', 'HV_COMMENT'):
             comments = str(v).split('\n')
             for com in comments:
