@@ -19,7 +19,7 @@ from matplotlib.figure import Figure
 
 import astropy.units as u
 import astropy.wcs
-from astropy.coordinates import Latitude, Longitude, SkyCoord, UnitSphericalRepresentation
+from astropy.coordinates import Longitude, SkyCoord, UnitSphericalRepresentation
 from astropy.nddata import NDData
 from astropy.visualization import AsymmetricPercentileInterval, HistEqStretch, ImageNormalize
 from astropy.visualization.wcsaxes import WCSAxes
@@ -35,16 +35,19 @@ from sunpy.image.resample import resample as sunpy_image_resample
 from sunpy.image.resample import reshape_image_to_4d_superpixel
 from sunpy.sun import constants
 from sunpy.time import is_time, parse_time
-from sunpy.util import expand_list
-from sunpy.util.decorators import deprecate_positional_args_since
-from sunpy.util.exceptions import SunpyUserWarning
+from sunpy.util import MetaDict, expand_list
+from sunpy.util.decorators import cached_property_based_on, deprecated
+from sunpy.util.exceptions import SunpyDeprecationWarning, SunpyMetadataWarning, SunpyUserWarning
 from sunpy.util.functools import seconddispatch
 from sunpy.visualization import axis_labels_from_ctype, peek_show, wcsaxes_compat
+from sunpy.visualization.colormaps import cm as sunpy_cm
 
 TIME_FORMAT = config.get("general", "time_format")
 PixelPair = namedtuple('PixelPair', 'x y')
 SpatialPair = namedtuple('SpatialPair', 'axis1 axis2')
 
+_META_FIX_URL = 'https://docs.sunpy.org/en/stable/code_ref/map.html#fixing-map-metadata'
+
 __all__ = ['GenericMap']
 
 
@@ -77,21 +80,21 @@ class GenericMap(NDData):
     >>> import sunpy.data.sample  # doctest: +REMOTE_DATA
     >>> aia = sunpy.map.Map(sunpy.data.sample.AIA_171_IMAGE)  # doctest: +REMOTE_DATA
     >>> aia   # doctest: +REMOTE_DATA
-    <sunpy.map.sources.sdo.AIAMap object at 0x...>
+    <sunpy.map.sources.sdo.AIAMap object at ...>
     SunPy Map
     ---------
-    Observatory:		 SDO
-    Instrument:		 AIA 3
-    Detector:		 AIA
-    Measurement:		 171.0 Angstrom
-    Wavelength:		 171.0 Angstrom
-    Observation Date:	 2011-06-07 06:33:02
-    Exposure Time:		 0.234256 s
-    Dimension:		 [1024. 1024.] pix
-    Coordinate System:	 helioprojective
-    Scale:			 [2.402792 2.402792] arcsec / pix
-    Reference Pixel:	 [512.5 512.5] pix
-    Reference Coord:	 [3.22309951 1.38578135] arcsec
+    Observatory:                 SDO
+    Instrument:          AIA 3
+    Detector:            AIA
+    Measurement:                 171.0 Angstrom
+    Wavelength:          171.0 Angstrom
+    Observation Date:    2011-06-07 06:33:02
+    Exposure Time:               0.234256 s
+    Dimension:           [1024. 1024.] pix
+    Coordinate System:   helioprojective
+    Scale:                       [2.402792 2.402792] arcsec / pix
+    Reference Pixel:     [511.5 511.5] pix
+    Reference Coord:     [3.22309951 1.38578135] arcsec
     array([[ -95.92475  ,    7.076416 ,   -1.9656711, ..., -127.96519  ,
             -127.96519  , -127.96519  ],
            [ -96.97533  ,   -5.1167884,    0.       , ...,  -98.924576 ,
@@ -183,7 +186,7 @@ class GenericMap(NDData):
             warnings.warn("This file contains more than 2 dimensions. "
                           "Data will be truncated to the first two dimensions.", SunpyUserWarning)
 
-        super().__init__(data, meta=header, **kwargs)
+        super().__init__(data, meta=MetaDict(header), **kwargs)
 
         # Correct possibly missing meta keywords
         self._fix_date()
@@ -220,6 +223,16 @@ class GenericMap(NDData):
         if plot_settings:
             self.plot_settings.update(plot_settings)
 
+        # Try and set the colormap. This is not always possible if this method
+        # is run before map sources fix some of their metadata, so
+        # just ignore any exceptions raised.
+        try:
+            cmap = self._get_cmap_name()
+            if cmap in sunpy_cm.cmlist:
+                self.plot_settings['cmap'] = cmap
+        except Exception:
+            pass
+
     def __getitem__(self, key):
         """ This should allow indexing by physical coordinate """
         raise NotImplementedError(
@@ -254,8 +267,11 @@ class GenericMap(NDData):
                                                     self._reference_latitude)),
                                tmf=TIME_FORMAT)
 
+    def __str__(self):
+        return f"{self._text_summary()}\n{self.data.__repr__()}"
+
     def __repr__(self):
-        return object.__repr__(self) + "\n" + self._text_summary() + "\n" + self.data.__repr__()
+        return f"{object.__repr__(self)}\n{self}"
 
     def _repr_html_(self):
         """
@@ -323,16 +339,42 @@ class GenericMap(NDData):
         if fig.canvas is None:
             FigureCanvasBase(fig)
         ax = fig.subplots()
-        ax.hist(finite_data.ravel(), bins=100, histtype='stepfilled')
+        values, bins, patches = ax.hist(finite_data.ravel(), bins=100)
+        norm_centers = norm(0.5 * (bins[:-1] + bins[1:])).data
+        for c, p in zip(norm_centers, patches):
+            plt.setp(p, "facecolor", cmap(c))
+        ax.plot(np.array([bins[:-1], bins[1:]]).T.ravel(),
+                np.array([values, values]).T.ravel())
         ax.set_facecolor('white')
         ax.semilogy()
         # Explicitly set the power limits for the X axis formatter to avoid text overlaps
         ax.xaxis.get_major_formatter().set_powerlimits((-3, 4))
-        ax.set_xlabel('Pixel value')
+        ax.set_xlabel('Pixel value in linear bins')
         ax.set_ylabel('# of pixels')
-        ax.set_title('Distribution of pixels with finite values')
+        ax.set_title('Distribution of pixel values [click for cumulative]')
         hist_src = _figure_to_base64(fig)
 
+        # Plot the CDF of the pixel values using a symmetric-log horizontal scale
+        fig = Figure(figsize=(4.8, 2.4), constrained_layout=True)
+        # TODO: Figure instances in matplotlib<3.1 do not create a canvas by default
+        if fig.canvas is None:
+            FigureCanvasBase(fig)
+        ax = fig.subplots()
+        n_bins = 256
+        bins = norm.inverse(np.arange(n_bins + 1) / n_bins)
+        values, _, patches = ax.hist(finite_data.ravel(), bins=bins, cumulative=True)
+        for i, p in enumerate(patches):
+            plt.setp(p, "facecolor", cmap((i + 0.5) / n_bins))
+        ax.plot(np.array([bins[:-1], bins[1:]]).T.ravel(),
+                np.array([values, values]).T.ravel())
+        ax.set_facecolor('white')
+        ax.set_xscale('symlog')
+        ax.set_yscale('log')
+        ax.set_xlabel('Pixel value in equalized bins')
+        ax.set_ylabel('Cumulative # of pixels')
+        ax.set_title('Cumulative distribution of pixel values')
+        cdf_src = _figure_to_base64(fig)
+
         return textwrap.dedent(f"""\
             <pre>{html.escape(object.__repr__(self))}</pre>
             <table>
@@ -357,7 +399,13 @@ class GenericMap(NDData):
                 <tr>
                 </tr>
                 <tr>
-                    <td><img src='data:image/png;base64,{hist_src}'/></td>
+                    <td><img src='data:image/png;base64,{hist_src}'
+                             src2='data:image/png;base64,{cdf_src}'
+                             onClick='var temp = this.src;
+                                      this.src = this.getAttribute("src2");
+                                      this.setAttribute("src2", temp)'
+                        />
+                    </td>
                 </tr>
             </table>""")
 
@@ -417,6 +465,11 @@ class GenericMap(NDData):
         return r.lon.to(self.spatial_units[0]), r.lat.to(self.spatial_units[1])
 
     @property
+    def _meta_hash(self):
+        return self.meta.item_hash()
+
+    @property
+    @cached_property_based_on('_meta_hash')
     def wcs(self):
         """
         The `~astropy.wcs.WCS` property of the map.
@@ -454,7 +507,7 @@ class GenericMap(NDData):
         w2.wcs.pc = self.rotation_matrix
         w2.wcs.cunit = self.spatial_units
         w2.wcs.dateobs = self.date.isot
-        w2.rsun = self.rsun_meters
+        w2.wcs.aux.rsun_ref = self.rsun_meters.to_value(u.m)
 
         # Astropy WCS does not understand the SOHO default of "solar-x" and
         # "solar-y" ctypes.  This overrides the default assignment and
@@ -466,12 +519,23 @@ class GenericMap(NDData):
         if w2.wcs.ctype[1].lower() in ("solar-y", "solar_y"):
             w2.wcs.ctype[1] = 'HPLT-TAN'
 
-        # GenericMap.coordinate_frame is implemented using this method, so we
-        # need to do this only based on .meta.
-        ctypes = {c[:4] for c in w2.wcs.ctype}
-        # Check that the ctypes contains one of these three pairs of axes.
-        if ({'HPLN', 'HPLT'} <= ctypes or {'SOLX', 'SOLY'} <= ctypes or {'CRLN', 'CRLT'} <= ctypes):
-            w2.heliographic_observer = self.observer_coordinate
+        # Set observer coordinate information
+        #
+        # Clear all the aux information that was set earlier. This is to avoid
+        # issues with maps that store multiple observer coordinate keywords.
+        # Note that we have to create a new WCS as it's not possible to modify
+        # wcs.wcs.aux in place.
+        header = w2.to_header()
+        for kw in ['crln_obs', 'dsun_obs', 'hgln_obs', 'hglt_obs']:
+            header.pop(kw, None)
+        w2 = astropy.wcs.WCS(header)
+
+        # Get observer coord, and transform if needed
+        obs_coord = self.observer_coordinate
+        if not isinstance(obs_coord.frame, (HeliographicStonyhurst, HeliographicCarrington)):
+            obs_coord = obs_coord.transform_to(HeliographicStonyhurst(obstime=self.date))
+
+        sunpy.coordinates.wcs_utils._set_wcs_aux_obs_coord(w2, obs_coord)
 
         # Validate the WCS here.
         w2.wcs.set()
@@ -480,7 +544,7 @@ class GenericMap(NDData):
     @property
     def coordinate_frame(self):
         """
-        An `astropy.coordinates.BaseFrame` instance created from the coordinate
+        An `astropy.coordinates.BaseCoordinateFrame` instance created from the coordinate
         information for this Map, or None if the frame cannot be determined.
         """
         try:
@@ -534,6 +598,7 @@ class GenericMap(NDData):
         return self.data.dtype
 
     @property
+    @deprecated(since="2.1", message="Use map.data.size instead", alternative="map.data.size")
     def size(self):
         """
         The number of pixels in the array of the map.
@@ -571,6 +636,30 @@ class GenericMap(NDData):
         """
         return self.data.max(*args, **kwargs)
 
+    @property
+    def unit(self):
+        """
+        Unit of the map data.
+
+        This is taken from the 'BUNIT' FITS keyword. If no 'BUNIT' entry is
+        present in the metadata then this returns `None`. If the 'BUNIT' value
+        cannot be parsed into a unit a warning is raised, and `None` returned.
+        """
+        unit_str = self.meta.get('bunit', None)
+        if unit_str is None:
+            return
+
+        unit = u.Unit(unit_str, format='fits', parse_strict='silent')
+        if isinstance(unit, u.UnrecognizedUnit):
+            warnings.warn(f'Could not parse unit string "{unit_str}" as a valid FITS unit.\n'
+                          f'See {_META_FIX_URL} for how to fix metadata before loading it '
+                          'with sunpy.map.Map.\n'
+                          'See https://fits.gsfc.nasa.gov/fits_standard.html for'
+                          'the FITS unit standards.',
+                          SunpyMetadataWarning)
+            unit = None
+        return unit
+
 # #### Keyword attribute and other attribute definitions #### #
 
     def _base_name(self):
@@ -611,6 +700,20 @@ class GenericMap(NDData):
         This is taken from the 'DATE-OBS' FITS keyword.
         """
         time = self.meta.get('date-obs', None)
+        # Get the time scale
+        if time is not None and 'TAI' in time:
+            # SDO specifies the 'TAI' scale in their time string, which is parsed
+            # by parse_time(). If a different timescale is also present, warn the
+            # user that it will be ignored.
+            timesys = 'TAI'
+            timesys_meta = self.meta.get('timesys', '').upper()
+            if timesys_meta != 'TAI':
+                warnings.warn('Found "TAI" in time string, ignoring TIMESYS keyword '
+                              f'which is set to "{timesys_meta}".', SunpyUserWarning)
+        else:
+            # UTC is the FITS standard default
+            timesys = self.meta.get('timesys', 'UTC')
+
         if time is None:
             if self._default_time is None:
                 warnings.warn("Missing metadata for observation time, "
@@ -619,7 +722,8 @@ class GenericMap(NDData):
                               SunpyUserWarning)
                 self._default_time = parse_time('now')
             time = self._default_time
-        return parse_time(time)
+
+        return parse_time(time, scale=timesys.lower())
 
     @property
     def detector(self):
@@ -630,6 +734,16 @@ class GenericMap(NDData):
         """
         return self.meta.get('detector', "")
 
+    @property
+    def timeunit(self):
+        """
+        The `~astropy.units.Unit` of the exposure time of this observation.
+
+        Taken from the "TIMEUNIT" FITS keyword, and defaults to seconds (as per)
+        the FITS standard).
+        """
+        return u.Unit(self.meta.get('timeunit', 's'))
+
     @property
     def exposure_time(self):
         """
@@ -637,7 +751,7 @@ class GenericMap(NDData):
 
         This is taken from the 'EXPTIME' FITS keyword.
         """
-        return self.meta.get('exptime', 0.0) * u.s
+        return self.meta.get('exptime', 0.0) * self.timeunit
 
     @property
     def instrument(self):
@@ -798,9 +912,24 @@ class GenericMap(NDData):
 
     @property
     def coordinate_system(self):
-        """Coordinate system used for x and y axes (ctype1/2)."""
-        return SpatialPair(self.meta.get('ctype1', 'HPLN-   '),
-                           self.meta.get('ctype2', 'HPLT-   '))
+        """
+        Coordinate system used for x and y axes (ctype1/2).
+
+        If not present, defaults to (HPLN-TAN, HPLT-TAN), and emits a warning.
+        """
+        ctype1 = self.meta.get('ctype1', None)
+        if ctype1 is None:
+            warnings.warn("Missing CTYPE1 from metadata, assuming CTYPE1 is HPLN-TAN",
+                          SunpyUserWarning)
+            ctype1 = 'HPLN-TAN'
+
+        ctype2 = self.meta.get('ctype2', None)
+        if ctype2 is None:
+            warnings.warn("Missing CTYPE2 from metadata, assuming CTYPE2 is HPLT-TAN",
+                          SunpyUserWarning)
+            ctype2 = 'HPLT-TAN'
+
+        return SpatialPair(ctype1, ctype2)
 
     @property
     def _supported_observer_coordinates(self):
@@ -842,22 +971,10 @@ class GenericMap(NDData):
             if all(meta_list):
                 sc = SkyCoord(obstime=self.date, **kwargs)
 
-                # We need to specially handle an observer location provided in Carrington
-                # coordinates.  To create the observer coordinate, we need to specify the
-                # frame, but defining a Carrington frame normally requires specifying the
-                # frame's observer.  This loop is the problem.  Instead, since the
-                # Carrington frame needs only the Sun-observer distance component from the
-                # frame's observer, we create the same frame using a fake observer that has
-                # the same Sun-observer distance.
+                # If the observer location is supplied in Carrington coordinates,
+                # the coordinate's `observer` attribute should be set to "self"
                 if isinstance(sc.frame, HeliographicCarrington):
-                    fake_observer = HeliographicStonyhurst(0*u.deg, 0*u.deg, sc.radius,
-                                                           obstime=sc.obstime)
-                    fake_frame = sc.frame.replicate(observer=fake_observer)
-                    hgs = fake_frame.transform_to(HeliographicStonyhurst(obstime=sc.obstime))
-
-                    # HeliographicStonyhurst doesn't need an observer, but adding the observer
-                    # facilitates a conversion back to HeliographicCarrington
-                    return SkyCoord(hgs, observer=hgs)
+                    sc.frame._observer = "self"
 
                 return sc.heliographic_stonyhurst
             elif any(meta_list) and not set(keys).isdisjoint(self.meta.keys()):
@@ -868,7 +985,7 @@ class GenericMap(NDData):
         warning_message = "".join(
             [f"For frame '{frame}' the following metadata is missing: {','.join(keys)}\n" for frame, keys in missing_meta.items()])
         warning_message = "Missing metadata for observer: assuming Earth-based observer.\n" + warning_message
-        warnings.warn(warning_message, SunpyUserWarning)
+        warnings.warn(warning_message, SunpyMetadataWarning, stacklevel=3)
 
         return get_earth(self.date)
 
@@ -1054,8 +1171,7 @@ class GenericMap(NDData):
 
         if err_message:
             err_message.append(
-                'See https://docs.sunpy.org/en/stable/code_ref/map.html#fixing-map-metadata` for '
-                'instructions on how to add missing metadata.')
+                f'See {_META_FIX_URL} for instructions on how to add missing metadata.')
             raise MapMetaValidationError('\n'.join(err_message))
 
         for meta_property in ('waveunit', ):
@@ -1075,21 +1191,27 @@ class GenericMap(NDData):
     @staticmethod
     def _check_origin(origin):
         """
-        Check origin is valid.
+        Check origin is valid, and raise a deprecation warning if it's not None.
         """
+        if origin is not None:
+            warnings.warn('The origin argument is deprecated. If using origin=1, '
+                          'manually subtract 1 from your pixels do not pass a value for origin.',
+                          SunpyDeprecationWarning)
         if origin not in [None, 0, 1]:
             raise ValueError('origin must be 0 or 1.')
 
-    def world_to_pixel(self, coordinate, origin=0):
+    def world_to_pixel(self, coordinate, origin=None):
         """
         Convert a world (data) coordinate to a pixel coordinate.
 
         Parameters
         ----------
-        coordinate : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseFrame`
+        coordinate : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
             The coordinate object to convert to pixel coordinates.
 
         origin : int
+            Deprecated.
+
             Origin of the top-left corner. i.e. count from 0 or 1.
             Normally, origin should be 0 when passing numpy indices, or 1 if
             passing values from FITS header or map attributes.
@@ -1102,11 +1224,6 @@ class GenericMap(NDData):
         y : `~astropy.units.Quantity`
             Pixel coordinate on the CTYPE2 axis.
         """
-        if not isinstance(coordinate, (SkyCoord,
-                                       astropy.coordinates.BaseCoordinateFrame)):
-            raise ValueError(
-                "world_to_pixel takes a Astropy coordinate frame or SkyCoord instance.")
-
         self._check_origin(origin)
         x, y = self.wcs.world_to_pixel(coordinate)
         if origin == 1:
@@ -1116,7 +1233,7 @@ class GenericMap(NDData):
         return PixelPair(x * u.pixel, y * u.pixel)
 
     @u.quantity_input
-    def pixel_to_world(self, x: u.pixel, y: u.pixel, origin=0):
+    def pixel_to_world(self, x: u.pixel, y: u.pixel, origin=None):
         """
         Convert a pixel coordinate to a data (world) coordinate.
 
@@ -1129,6 +1246,8 @@ class GenericMap(NDData):
             Pixel coordinate of the CTYPE2 axis. (Normally solar-y).
 
         origin : int
+            Deprecated.
+
             Origin of the top-left corner. i.e. count from 0 or 1.
             Normally, origin should be 0 when passing numpy indices, or 1 if
             passing values from FITS header or map attributes.
@@ -1261,7 +1380,7 @@ class GenericMap(NDData):
         ----------
         angle : `~astropy.units.Quantity`
             The angle (degrees) to rotate counterclockwise.
-        rmatrix : ndarray
+        rmatrix : array-like
             2x2 linear transformation rotation matrix.
         order : int
             Interpolation order to be used. Must be in the range 0-5.
@@ -1295,8 +1414,8 @@ class GenericMap(NDData):
 
         See Also
         --------
-        sunpy.image.transform.affine_transform : The routine this method calls
-        for the rotation.
+        sunpy.image.transform.affine_transform :
+            The routine this method calls for the rotation.
 
         Notes
         -----
@@ -1357,8 +1476,7 @@ class GenericMap(NDData):
         temp_map = self._new_instance(new_data, new_meta, self.plot_settings)
 
         # Convert the axis of rotation from data coordinates to pixel coordinates
-        pixel_rotation_center = u.Quantity(temp_map.world_to_pixel(self.reference_coordinate,
-                                                                   origin=0)).value
+        pixel_rotation_center = u.Quantity(temp_map.world_to_pixel(self.reference_coordinate)).value
         del temp_map
 
         if recenter:
@@ -1427,7 +1545,6 @@ class GenericMap(NDData):
 
         return new_map
 
-    @deprecate_positional_args_since(since='2.0', keyword_only=('width', 'height'))
     @u.quantity_input
     def submap(self, bottom_left, *, top_right=None, width: (u.deg, u.pix) = None, height: (u.deg, u.pix) = None):
         """
@@ -1444,12 +1561,10 @@ class GenericMap(NDData):
             The bottom-left coordinate of the rectangle. If a `~astropy.coordinates.SkyCoord` it can
             have shape ``(2,)`` and simultaneously define ``top_right``. If specifying
             pixel coordinates it must be given as an `~astropy.units.Quantity`
-            object with units of `~astropy.units.pix`.
+            object with units of `~astropy.units.si.pix`.
         top_right : `astropy.units.Quantity` or `~astropy.coordinates.SkyCoord`, optional
             The top-right coordinate of the rectangle. If ``top_right`` is
             specified ``width`` and ``height`` must be omitted.
-            Passing this as a positional argument is deprecated, you must pass
-            it as ``top_right=...``.
         width : `astropy.units.Quantity`, optional
             The width of the rectangle. Required if ``top_right`` is omitted.
         height : `astropy.units.Quantity`
@@ -1477,7 +1592,7 @@ class GenericMap(NDData):
         >>> bl = SkyCoord(-300*u.arcsec, -300*u.arcsec, frame=aia.coordinate_frame)  # doctest: +REMOTE_DATA
         >>> tr = SkyCoord(500*u.arcsec, 500*u.arcsec, frame=aia.coordinate_frame)  # doctest: +REMOTE_DATA
         >>> aia.submap(bl, top_right=tr)   # doctest: +REMOTE_DATA
-        <sunpy.map.sources.sdo.AIAMap object at 0x...>
+        <sunpy.map.sources.sdo.AIAMap object at ...>
         SunPy Map
         ---------
         Observatory:         SDO
@@ -1494,8 +1609,8 @@ class GenericMap(NDData):
         Reference Coord:     [3.22309951 1.38578135] arcsec
         ...
 
-        >>> aia.submap([0,0]*u.pixel, [5,5]*u.pixel)   # doctest: +REMOTE_DATA
-        <sunpy.map.sources.sdo.AIAMap object at 0x...>
+        >>> aia.submap([0,0]*u.pixel, top_right=[5,5]*u.pixel)   # doctest: +REMOTE_DATA
+        <sunpy.map.sources.sdo.AIAMap object at ...>
         SunPy Map
         ---------
         Observatory:         SDO
@@ -1515,7 +1630,7 @@ class GenericMap(NDData):
         >>> width = 10 * u.arcsec
         >>> height = 10 * u.arcsec
         >>> aia.submap(bl, width=width, height=height)   # doctest: +REMOTE_DATA
-        <sunpy.map.sources.sdo.AIAMap object at 0x7f91aecc5438>
+        <sunpy.map.sources.sdo.AIAMap object at ...>
         SunPy Map
         ---------
         Observatory:         SDO
@@ -1534,7 +1649,7 @@ class GenericMap(NDData):
 
         >>> bottom_left_vector = SkyCoord([0, 10]  * u.deg, [0, 10] * u.deg, frame='heliographic_stonyhurst')
         >>> aia.submap(bottom_left_vector)   # doctest: +REMOTE_DATA
-        <sunpy.map.sources.sdo.AIAMap object at 0x7f91aece8be0>
+        <sunpy.map.sources.sdo.AIAMap object at ...>
         SunPy Map
         ---------
         Observatory:         SDO
@@ -1557,7 +1672,8 @@ class GenericMap(NDData):
                 not in [[True, False, False], [False, False, False], [False, True, True]]):
             raise ValueError("Either top_right alone or both width and height must be specified.")
         # parse input arguments
-        pixel_corners = u.Quantity(self._parse_submap_input(bottom_left, top_right, width, height)).T
+        pixel_corners = u.Quantity(self._parse_submap_input(
+            bottom_left, top_right, width, height)).T
 
         # The pixel corners result is in Cartesian order, so the first index is
         # columns and the second is rows.
@@ -1840,11 +1956,10 @@ class GenericMap(NDData):
         c_kw.update(kwargs)
 
         circ = patches.Circle([0, 0], **c_kw)
-        axes.add_artist(circ)
+        axes.add_patch(circ)
 
         return [circ]
 
-    @deprecate_positional_args_since(since='2.0')
     @u.quantity_input
     def draw_rectangle(self, bottom_left, *, width: u.deg = None, height: u.deg = None,
                        axes=None, top_right=None, **kwargs):
@@ -1865,13 +1980,10 @@ class GenericMap(NDData):
             have shape ``(2,)`` to simultaneously define ``top_right``.
         top_right : `~astropy.coordinates.SkyCoord`
             The top-right coordinate of the rectangle.
-            Passing this as a positional argument is deprecated.
         width : `astropy.units.Quantity`, optional
             The width of the rectangle. Required if ``top_right`` is omitted.
-            Passing this as a positional argument is deprecated.
         height : `astropy.units.Quantity`
             The height of the rectangle. Required if ``top_right`` is omitted.
-            Passing this as a positional argument is deprecated.
         axes : `matplotlib.axes.Axes`
             The axes on which to plot the rectangle, defaults to the current
             axes.
@@ -1887,19 +1999,16 @@ class GenericMap(NDData):
         Extra keyword arguments to this function are passed through to the
         `~matplotlib.patches.Rectangle` instance.
         """
-        if isinstance(top_right, u.Quantity) and isinstance(width, u.Quantity):
-            # The decorator assigns the first positional arg to top_right and so on.
-            height = width
-            width = top_right
-            top_right = None
-
         bottom_left, top_right = get_rectangle_coordinates(bottom_left,
                                                            top_right=top_right,
                                                            width=width,
                                                            height=height)
 
+        bottom_left = bottom_left.transform_to(self.coordinate_frame)
+        top_right = top_right.transform_to(self.coordinate_frame)
+
         width = Longitude(top_right.spherical.lon - bottom_left.spherical.lon)
-        height = Latitude(top_right.spherical.lat - bottom_left.spherical.lat)
+        height = top_right.spherical.lat - bottom_left.spherical.lat
 
         if not axes:
             axes = plt.gca()
@@ -1908,17 +2017,16 @@ class GenericMap(NDData):
         else:
             axes_unit = self.spatial_units[0]
 
-        coord = bottom_left.transform_to(self.coordinate_frame)
-        bottom_left = u.Quantity((coord.spherical.lon, coord.spherical.lat), unit=axes_unit).value
+        bottom_left = u.Quantity(self._get_lon_lat(bottom_left), unit=axes_unit).value
 
         width = width.to(axes_unit).value
         height = height.to(axes_unit).value
         kwergs = {'transform': wcsaxes_compat.get_world_transform(axes),
-                  'color': 'white',
+                  'edgecolor': 'white',
                   'fill': False}
         kwergs.update(kwargs)
         rect = plt.Rectangle(bottom_left, width, height, **kwergs)
-        axes.add_artist(rect)
+        axes.add_patch(rect)
         return [rect]
 
     @u.quantity_input
@@ -2066,7 +2174,6 @@ class GenericMap(NDData):
 
         # Normal plot
         plot_settings = copy.deepcopy(self.plot_settings)
-
         if 'title' in plot_settings:
             plot_settings_title = plot_settings.pop('title')
         else:
@@ -2107,6 +2214,19 @@ class GenericMap(NDData):
             imshow_args['vmin'] = vmin
             imshow_args['vmax'] = vmax
 
+        if 'norm' in imshow_args:
+            norm = imshow_args['norm']
+            if 'vmin' in imshow_args:
+                if norm.vmin is not None:
+                    raise ValueError('Cannot manually specify vmin, as the norm '
+                                     'already has vmin set')
+                norm.vmin = imshow_args.pop('vmin')
+            if 'vmax' in imshow_args:
+                if norm.vmax is not None:
+                    raise ValueError('Cannot manually specify vmax, as the norm '
+                                     'already has vmax set')
+                norm.vmax = imshow_args.pop('vmax')
+
         if self.mask is None:
             ret = axes.imshow(self.data, **imshow_args)
         else:
@@ -2123,6 +2243,57 @@ class GenericMap(NDData):
 
         return ret
 
+    def contour(self, level, **kwargs):
+        """
+        Returns coordinates of the contours for a given level value.
+
+        For details of the contouring algorithm see `skimage.measure.find_contours`.
+
+        Parameters
+        ----------
+        level : float, astropy.units.Quantity
+            Value along which to find contours in the array. If the map unit attribute
+            is not `None`, this must be a `~astropy.units.Quantity` with units
+            equivalent to the map data units.
+        kwargs :
+            Additional keyword arguments are passed to `skimage.measure.find_contours`.
+
+        Returns
+        -------
+        contours: list of (n,2) `~astropy.coordinates.SkyCoord`
+            Coordinates of each contour.
+
+        Examples
+        --------
+        >>> import astropy.units as u
+        >>> import sunpy.map
+        >>> import sunpy.data.sample  # doctest: +REMOTE_DATA
+        >>> aia = sunpy.map.Map(sunpy.data.sample.AIA_171_IMAGE)  # doctest: +REMOTE_DATA
+        >>> contours = aia.contour(50000 * u.ct)  # doctest: +REMOTE_DATA
+        >>> print(contours[0])  # doctest: +REMOTE_DATA
+            <SkyCoord (Helioprojective: obstime=2011-06-07T06:33:02.770, rsun=696000000.0 m, observer=<HeliographicStonyhurst Coordinate (obstime=2011-06-07T06:33:02.770): (lon, lat, radius) in (deg, deg, m)
+        (-0.00406308, 0.04787238, 1.51846026e+11)>): (Tx, Ty) in arcsec
+        [(719.59798458, -352.60839064), (717.19243987, -353.75348121),
+        ...
+
+        See also
+        --------
+        `skimage.measure.find_contours`
+        """
+        from skimage import measure
+
+        if self.unit is not None:
+            try:
+                level = level.to_value(self.unit)
+            # Catch cases where level can't be converted or isn't a Quantity
+            except (AttributeError, u.UnitConversionError) as e:
+                raise u.UnitsError(
+                    f'level must be an astropy quantity convertible to {self.unit}') from e
+
+        contours = measure.find_contours(self.data, level=level, **kwargs)
+        contours = [self.wcs.array_index_to_world(c[:, 0], c[:, 1]) for c in contours]
+        return contours
+
 
 class InvalidHeaderInformation(ValueError):
     """Exception to raise when an invalid header tag value is encountered for a
