@@ -11,7 +11,7 @@ import pytest
 
 import astropy.units as u
 import astropy.wcs
-from astropy.coordinates import SkyCoord
+from astropy.coordinates import Latitude, SkyCoord
 from astropy.io import fits
 from astropy.io.fits.verify import VerifyWarning
 from astropy.tests.helper import assert_quantity_allclose
@@ -26,97 +26,11 @@ import sunpy.sun
 from sunpy.coordinates import sun
 from sunpy.time import parse_time
 from sunpy.util import SunpyDeprecationWarning, SunpyUserWarning
+from sunpy.util.exceptions import SunpyMetadataWarning
 
 testpath = sunpy.data.test.rootdir
 
 
-@pytest.fixture
-def test_map(request):
-    return request.getfixturevalue(request.param)
-
-
-@pytest.fixture
-def hmi_test_map():
-    return sunpy.map.Map(os.path.join(testpath, "resampled_hmi.fits"))
-
-
-@pytest.fixture
-def aia171_test_map():
-    map = sunpy.map.Map(os.path.join(testpath, 'aia_171_level1.fits'))
-
-    # Get rid of the blank keyword to prevent some astropy fits fixing warnings
-    header = dict(map.meta)
-    header.pop('blank')
-    return sunpy.map.Map((map.data, header))
-
-
-@pytest.fixture
-def aia171_roll_map(aia171_test_map):
-    return aia171_test_map.rotate(-45*u.deg)
-
-
-@pytest.fixture
-def heliographic_test_map():
-    map = sunpy.map.Map(os.path.join(testpath, 'heliographic_phase_map.fits.gz'))
-
-    # Fix unit strings to prevent some astropy fits fixing warnings
-    header = dict(map.meta)
-    header['cunit1'] = 'deg'
-    header['cunit2'] = 'deg'
-    return sunpy.map.Map((map.data, header))
-
-
-@pytest.fixture
-def aia171_test_map_with_mask(aia171_test_map):
-    shape = aia171_test_map.data.shape
-    mask = np.zeros_like(aia171_test_map.data, dtype=bool)
-    mask[0:shape[0] // 2, 0:shape[1] // 2] = True
-    return sunpy.map.Map(np.ma.array(aia171_test_map.data, mask=mask), aia171_test_map.meta)
-
-
-@pytest.fixture
-def generic_map():
-    data = np.ones([6, 6], dtype=np.float64)
-    header = {
-        'CRVAL1': 0,
-        'CRVAL2': 0,
-        'CRPIX1': 5,
-        'CRPIX2': 5,
-        'CDELT1': 10,
-        'CDELT2': 10,
-        'CUNIT1': 'arcsec',
-        'CUNIT2': 'arcsec',
-        'CTYPE1': 'HPLN-TAN',
-        'CTYPE2': 'HPLT-TAN',
-        'PC1_1': 0,
-        'PC1_2': -1,
-        'PC2_1': 1,
-        'PC2_2': 0,
-        'NAXIS1': 6,
-        'NAXIS2': 6,
-        'date-obs': '1970-01-01T00:00:00',
-        'mjd-obs': 40587.0,
-        'obsrvtry': 'Foo',
-        'detector': 'bar',
-        'wavelnth': 10,
-        'waveunit': 'm'
-    }
-    return sunpy.map.Map((data, header))
-
-
-@pytest.fixture
-def simple_map():
-    # A 3x3 map, with it's center at (0, 0), and scaled differently in
-    # each direction
-    data = np.arange(9).reshape((3, 3))
-    ref_coord = SkyCoord(0.0, 0.0, frame='helioprojective', obstime='now', unit='deg')
-    ref_pix = [1, 1] * u.pix
-    scale = [2, 1] * u.arcsec / u.pix
-    header = sunpy.map.make_fitswcs_header(data, ref_coord, reference_pixel=ref_pix, scale=scale)
-
-    return sunpy.map.Map(data, header)
-
-
 def test_fits_data_comparison(aia171_test_map):
     """Make sure the data is the same in pyfits and SunPy"""
     with pytest.warns(VerifyWarning, match="Invalid 'BLANK' keyword in header."):
@@ -144,6 +58,22 @@ def test_wcs(aia171_test_map):
     np.testing.assert_allclose(wcs.wcs.pc, aia171_test_map.rotation_matrix)
 
 
+def test_wcs_cache(aia171_test_map):
+    wcs1 = aia171_test_map.wcs
+    wcs2 = aia171_test_map.wcs
+    # Check that without any changes to the header, retreiving the wcs twice
+    # returns the same object instead of recomputing the wcs
+    assert wcs1 is wcs2
+
+    # Change the header and make sure the wcs is re-computed
+    new_crpix = 20
+    assert new_crpix != wcs2.wcs.crpix[0]
+    aia171_test_map.meta['crpix1'] = new_crpix
+
+    new_wcs = aia171_test_map.wcs
+    assert new_wcs.wcs.crpix[0] == new_crpix
+
+
 def test_header_immutability(aia171_test_map):
     # Check that accessing the wcs of a map doesn't modify the meta data
     assert 'KEYCOMMENTS' in aia171_test_map.meta
@@ -156,7 +86,8 @@ def test_dtype(generic_map):
 
 
 def test_size(generic_map):
-    assert generic_map.size == 36 * u.pix
+    with pytest.warns(SunpyDeprecationWarning, match='Use map.data.size instead'):
+        assert generic_map.size == 36 * u.pix
 
 
 def test_min(generic_map):
@@ -175,6 +106,13 @@ def test_std(generic_map):
     assert generic_map.std() == 0
 
 
+def test_unit(generic_map):
+    assert generic_map.unit == u.ct / u.s
+    generic_map.meta['bunit'] = 'not a unit'
+    with pytest.warns(SunpyMetadataWarning, match='Could not parse unit string "not a unit"'):
+        assert generic_map.unit is None
+
+
 # ==============================================================================
 # Test the default value of a load of properties
 # TODO: Test the header keyword extraction
@@ -197,6 +135,14 @@ def test_date(generic_map):
     assert isinstance(generic_map.date, Time)
 
 
+def test_date_scale(generic_map):
+    # Check that default time scale is UTC
+    assert 'timesys' not in generic_map.meta
+    assert generic_map.date.scale == 'utc'
+    generic_map.meta['timesys'] = 'tai'
+    assert generic_map.date.scale == 'tai'
+
+
 def test_date_aia(aia171_test_map):
     assert aia171_test_map.date == parse_time('2011-02-15T00:00:00.34')
 
@@ -205,9 +151,14 @@ def test_detector(generic_map):
     assert generic_map.detector == 'bar'
 
 
+def test_timeunit(generic_map):
+    assert generic_map.timeunit == u.Unit('s')
+    generic_map.meta['timeunit'] = 'h'
+    assert generic_map.timeunit == u.Unit('h')
+
+
 def test_dsun(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer: assuming Earth-based observer.*'):
-        assert_quantity_allclose(generic_map.dsun, sun.earth_distance(generic_map.date))
+    assert_quantity_allclose(generic_map.dsun, sun.earth_distance(generic_map.date))
 
 
 def test_rsun_meters(generic_map):
@@ -225,19 +176,28 @@ def test_coordinate_system(generic_map):
     assert generic_map.coordinate_system == ('HPLN-TAN', 'HPLT-TAN')
 
 
+def test_default_coordinate_system(generic_map):
+    generic_map.meta.pop('ctype1')
+    with pytest.warns(SunpyUserWarning, match='Missing CTYPE1 from metadata'):
+        assert generic_map.coordinate_system == ('HPLN-TAN', 'HPLT-TAN')
+
+    generic_map.meta.pop('ctype2')
+    generic_map.meta['ctype1'] = 'HPLN-TAN'
+    with pytest.warns(SunpyUserWarning, match='Missing CTYPE2 from metadata'):
+        assert generic_map.coordinate_system == ('HPLN-TAN', 'HPLT-TAN')
+
+
 def test_carrington_longitude(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer: assuming Earth-based observer.*'):
-        assert generic_map.carrington_longitude == sun.L0(generic_map.date)
+    assert u.allclose(generic_map.carrington_longitude, sun.L0(generic_map.date))
 
 
 def test_heliographic_latitude(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer: assuming Earth-based observer.*'):
-        assert generic_map.heliographic_latitude == sun.B0(generic_map.date)
+    assert u.allclose(generic_map.heliographic_latitude, Latitude(sun.B0(generic_map.date)))
 
 
 def test_heliographic_longitude(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer: assuming Earth-based observer.*'):
-        assert generic_map.heliographic_longitude == 0.
+    # Needs a small tolerance to account for 32bit rounding errors
+    assert u.allclose(generic_map.heliographic_longitude, 0 * u.deg, atol=1e-15*u.deg)
 
 
 def test_units(generic_map):
@@ -265,7 +225,7 @@ def test_heliographic_longitude_crln(hmi_test_map):
 
 def test_remove_observers(aia171_test_map):
     aia171_test_map._remove_existing_observer_location()
-    with pytest.warns(SunpyUserWarning,
+    with pytest.warns(SunpyMetadataWarning,
                       match='Missing metadata for observer: assuming Earth-based observer.*'):
         aia171_test_map.observer_coordinate
 
@@ -275,7 +235,8 @@ def test_partially_missing_observers(generic_map):
     generic_map.meta['hgln_obs'] = 0
     generic_map.meta['crlt_obs'] = 0
     generic_map.meta['crln_obs'] = 0
-    with pytest.warns(SunpyUserWarning,
+    generic_map.meta.pop('dsun_obs')
+    with pytest.warns(SunpyMetadataWarning,
                       match="Missing metadata for observer: assuming Earth-based observer.\n" +
                             "For frame 'heliographic_stonyhurst' the following metadata is missing: dsun_obs\n" +
                             "For frame 'heliographic_carrington' the following metadata is missing: dsun_obs\n"):
@@ -312,7 +273,9 @@ def test_rotation_matrix_cd_cdelt():
         'NAXIS1': 6,
         'NAXIS2': 6,
         'CUNIT1': 'arcsec',
-        'CUNIT2': 'arcsec'
+        'CUNIT2': 'arcsec',
+        'CTYPE1': 'HPLN-TAN',
+        'CTYPE2': 'HPLT-TAN',
     }
     cd_map = sunpy.map.Map((data, header))
     np.testing.assert_allclose(cd_map.rotation_matrix, np.array([[0., -1.], [1., 0]]))
@@ -334,7 +297,9 @@ def test_rotation_matrix_cd_cdelt_square():
         'NAXIS1': 6,
         'NAXIS2': 6,
         'CUNIT1': 'arcsec',
-        'CUNIT2': 'arcsec'
+        'CUNIT2': 'arcsec',
+        'CTYPE1': 'HPLN-TAN',
+        'CTYPE2': 'HPLT-TAN',
     }
     cd_map = sunpy.map.Map((data, header))
     np.testing.assert_allclose(cd_map.rotation_matrix, np.array([[0., -1], [1., 0]]))
@@ -346,16 +311,13 @@ def test_swap_cd():
 
 
 def test_world_to_pixel(generic_map):
-    """Make sure conversion from data units to pixels is internally
-    consistent"""
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        # Note: FITS pixels start from 1,1
-        test_pixel = generic_map.world_to_pixel(generic_map.reference_coordinate)
+    """Make sure conversion from data units to pixels is internally consistent"""
+    test_pixel = generic_map.world_to_pixel(generic_map.reference_coordinate)
     assert_quantity_allclose(test_pixel, generic_map.reference_pixel)
 
 
 def test_world_to_pixel_error(generic_map):
-    strerr = 'world_to_pixel takes a Astropy coordinate frame or SkyCoord instance'
+    strerr = 'Expected the following order of world arguments: SkyCoord'
     with pytest.raises(ValueError, match=strerr):
         generic_map.world_to_pixel(1)
 
@@ -363,8 +325,9 @@ def test_world_to_pixel_error(generic_map):
 @pytest.mark.parametrize('origin', [0, 1])
 def test_world_pixel_roundtrip(simple_map, origin):
     pix = 1 * u.pix, 1 * u.pix
-    coord = simple_map.pixel_to_world(*pix, origin=origin)
-    pix_roundtrip = simple_map.world_to_pixel(coord, origin=origin)
+    with pytest.warns(SunpyDeprecationWarning, match='The origin argument is deprecated'):
+        coord = simple_map.pixel_to_world(*pix, origin=origin)
+        pix_roundtrip = simple_map.world_to_pixel(coord, origin=origin)
 
     assert u.allclose(pix_roundtrip.x, pix[0], atol=1e-10 * u.pix)
     assert u.allclose(pix_roundtrip.y, pix[1], atol=1e-10 * u.pix)
@@ -395,7 +358,10 @@ def test_save(aia171_test_map, generic_map):
     aiamap.save(afilename, filetype='fits', overwrite=True)
     loaded_save = sunpy.map.Map(afilename)
     assert isinstance(loaded_save, sunpy.map.sources.AIAMap)
-    assert loaded_save.meta == aiamap.meta
+    # Compare metadata without considering ordering of keys
+    assert loaded_save.meta.keys() == aiamap.meta.keys()
+    for k in aiamap.meta:
+        assert loaded_save.meta[k] == aiamap.meta[k]
     assert_quantity_allclose(loaded_save.data, aiamap.data)
 
 
@@ -428,6 +394,8 @@ def test_default_shift():
         'NAXIS2': 6,
         'CUNIT1': 'arcsec',
         'CUNIT2': 'arcsec',
+        'CTYPE1': 'HPLN-TAN',
+        'CTYPE2': 'HPLT-TAN',
     }
     cd_map = sunpy.map.Map((data, header))
     assert cd_map.shifted_value[0].value == 0
@@ -436,15 +404,13 @@ def test_default_shift():
 
 def test_shift_applied(generic_map):
     """Test that adding a shift actually updates the reference coordinate"""
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        original_reference_coord = (generic_map.reference_coordinate.Tx,
-                                    generic_map.reference_coordinate.Ty)
+    original_reference_coord = (generic_map.reference_coordinate.Tx,
+                                generic_map.reference_coordinate.Ty)
     x_shift = 5 * u.arcsec
     y_shift = 13 * u.arcsec
     shifted_map = generic_map.shift(x_shift, y_shift)
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        assert shifted_map.reference_coordinate.Tx - x_shift == original_reference_coord[0]
-        assert shifted_map.reference_coordinate.Ty - y_shift == original_reference_coord[1]
+    assert shifted_map.reference_coordinate.Tx - x_shift == original_reference_coord[0]
+    assert shifted_map.reference_coordinate.Ty - y_shift == original_reference_coord[1]
     crval1 = ((generic_map.meta.get('crval1') * generic_map.spatial_units[0] +
                shifted_map.shifted_value[0]).to(shifted_map.spatial_units[0])).value
     assert shifted_map.meta.get('crval1') == crval1
@@ -628,14 +594,14 @@ def test_resample(simple_map, shape):
     # Check that the corner coordinates of the input and output are the same
     resampled_lower_left = resampled.pixel_to_world(-0.5 * u.pix, -0.5 * u.pix)
     original_lower_left = simple_map.pixel_to_world(-0.5 * u.pix, -0.5 * u.pix)
-    assert resampled_lower_left.Tx == original_lower_left.Tx
-    assert resampled_lower_left.Ty == original_lower_left.Ty
+    assert u.allclose(resampled_lower_left.Tx, original_lower_left.Tx)
+    assert u.allclose(resampled_lower_left.Ty, original_lower_left.Ty)
 
     resampled_upper_left = resampled.pixel_to_world((shape[0] - 0.5) * u.pix,
                                                     (shape[1] - 0.5) * u.pix)
     original_upper_left = simple_map.pixel_to_world(2.5 * u.pix, 2.5 * u.pix)
-    assert resampled_upper_left.Tx == original_upper_left.Tx
-    assert resampled_upper_left.Ty == original_upper_left.Ty
+    assert u.allclose(resampled_upper_left.Tx, original_upper_left.Tx)
+    assert u.allclose(resampled_upper_left.Ty, original_upper_left.Ty)
 
 
 resample_test_data = [('linear', (100, 200) * u.pixel), ('neighbor', (128, 256) * u.pixel),
@@ -645,8 +611,7 @@ resample_test_data = [('linear', (100, 200) * u.pixel), ('neighbor', (128, 256)
 @pytest.mark.parametrize('sample_method, new_dimensions', resample_test_data)
 def test_resample_dimensions(generic_map, sample_method, new_dimensions):
     """Check that resampled map has expected dimensions."""
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        resampled_map = generic_map.resample(new_dimensions, method=sample_method)
+    resampled_map = generic_map.resample(new_dimensions, method=sample_method)
     assert resampled_map.dimensions[0] == new_dimensions[0]
     assert resampled_map.dimensions[1] == new_dimensions[1]
 
@@ -656,19 +621,17 @@ def test_resample_metadata(generic_map, sample_method, new_dimensions):
     """
     Check that the resampled map has correctly adjusted metadata.
     """
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        resampled_map = generic_map.resample(new_dimensions, method=sample_method)
+    resampled_map = generic_map.resample(new_dimensions, method=sample_method)
     assert float(resampled_map.meta['cdelt1']) / generic_map.meta['cdelt1'] \
         == float(generic_map.data.shape[1]) / resampled_map.data.shape[1]
     assert float(resampled_map.meta['cdelt2']) / generic_map.meta['cdelt2'] \
         == float(generic_map.data.shape[0]) / resampled_map.data.shape[0]
     assert resampled_map.meta['crpix1'] == (resampled_map.data.shape[1] + 1) / 2.
     assert resampled_map.meta['crpix2'] == (resampled_map.data.shape[0] + 1) / 2.
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        assert resampled_map.meta['crval1'] == generic_map.center.Tx.value
-        assert resampled_map.meta['crval2'] == generic_map.center.Ty.value
-        assert resampled_map.meta['naxis1'] == new_dimensions[0].value
-        assert resampled_map.meta['naxis2'] == new_dimensions[1].value
+    assert resampled_map.meta['crval1'] == generic_map.center.Tx.value
+    assert resampled_map.meta['crval2'] == generic_map.center.Ty.value
+    assert resampled_map.meta['naxis1'] == new_dimensions[0].value
+    assert resampled_map.meta['naxis2'] == new_dimensions[1].value
     for key in generic_map.meta:
         if key not in ('cdelt1', 'cdelt2', 'crpix1', 'crpix2', 'crval1',
                        'crval2', 'naxis1', 'naxis2'):
@@ -714,10 +677,10 @@ def test_superpixel(aia171_test_map, aia171_test_map_with_mask):
     superpixel_map_sum = aia171_test_map_with_mask.superpixel(dimensions, offset=(4, 4) * u.pix)
     assert_quantity_allclose(
         superpixel_map_sum.dimensions[0],
-        np.int((aia171_test_map.dimensions[0] / dimensions[0]).value) * u.pix - 1 * u.pix)
+        int((aia171_test_map.dimensions[0] / dimensions[0]).value) * u.pix - 1 * u.pix)
     assert_quantity_allclose(
         superpixel_map_sum.dimensions[1],
-        np.int((aia171_test_map.dimensions[1] / dimensions[1]).value) * u.pix - 1 * u.pix)
+        int((aia171_test_map.dimensions[1] / dimensions[1]).value) * u.pix - 1 * u.pix)
 
 
 def test_superpixel_err(generic_map):
@@ -778,8 +741,7 @@ def test_rotate(aia171_test_map):
 
 
 def test_rotate_pad_crpix(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        rotated_map = generic_map.rotate(30*u.deg)
+    rotated_map = generic_map.rotate(30*u.deg)
     # This tests that the reference pixel of the map is in the expected place.
     assert rotated_map.data.shape != generic_map.data.shape
     assert_quantity_allclose(u.Quantity(rotated_map.reference_pixel),
@@ -787,8 +749,7 @@ def test_rotate_pad_crpix(generic_map):
 
 
 def test_rotate_recenter(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        rotated_map = generic_map.rotate(20 * u.deg, recenter=True)
+    rotated_map = generic_map.rotate(20 * u.deg, recenter=True)
     pixel_array_center = (np.flipud(rotated_map.data.shape) - 1) / 2.0
 
     assert_quantity_allclose(
@@ -802,16 +763,14 @@ def test_rotate_crota_remove(aia171_test_map):
 
 
 def test_rotate_scale_cdelt(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        rot_map = generic_map.rotate(scale=10.)
+    rot_map = generic_map.rotate(scale=10.)
     assert rot_map.meta['CDELT1'] == generic_map.meta['CDELT1'] / 10.
     assert rot_map.meta['CDELT2'] == generic_map.meta['CDELT2'] / 10.
 
 
 def test_rotate_new_matrix(generic_map):
     # Rotate by CW90 to go from CCW 90 in generic map to CCW 180
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        rot_map = generic_map.rotate(rmatrix=np.array([[0, 1], [-1, 0]]))
+    rot_map = generic_map.rotate(rmatrix=np.array([[0, 1], [-1, 0]]))
     np.testing.assert_allclose(rot_map.rotation_matrix, np.array([[-1, 0], [0, -1]]))
 
 
@@ -830,19 +789,7 @@ def test_rotate_invalid_order(generic_map):
 def test_as_mpl_axes_aia171(aia171_test_map):
     ax = plt.subplot(projection=aia171_test_map)
     assert isinstance(ax, wcsaxes.WCSAxes)
-    # This test doesn't work, it seems that WCSAxes copies or changes the WCS
-    # object.
-    #  assert ax.wcs is aia171_test_map.wcs
     assert all([ct1 == ct2 for ct1, ct2 in zip(ax.wcs.wcs.ctype, aia171_test_map.wcs.wcs.ctype)])
-    # Map adds these attributes, so we use them to check.
-    assert hasattr(ax.wcs, 'heliographic_observer')
-
-
-def test_pixel_to_world_no_projection(generic_map):
-    with pytest.warns(SunpyUserWarning, match='Missing metadata for observer'):
-        out = generic_map.pixel_to_world(*u.Quantity(generic_map.reference_pixel))
-    assert_quantity_allclose(out.Tx, 0*u.arcsec, atol=1e-26*u.arcsec)
-    assert_quantity_allclose(out.Ty, 0*u.arcsec, atol=1e-26*u.arcsec)
 
 
 def test_validate_meta(generic_map):
@@ -957,10 +904,12 @@ def test_missing_metadata_warnings():
         header = {}
         header['cunit1'] = 'arcsec'
         header['cunit2'] = 'arcsec'
+        header['ctype1'] = 'HPLN-TAN'
+        header['ctype2'] = 'HPLT-TAN'
         array_map = sunpy.map.Map(np.random.rand(20, 15), header)
         array_map.peek()
     # There should be 2 warnings for missing metadata (obstime and observer location)
-    assert len([w for w in record if w.category is SunpyUserWarning]) == 2
+    assert len([w for w in record if w.category in (SunpyMetadataWarning, SunpyUserWarning)]) == 2
 
 
 def test_fits_header(aia171_test_map):
@@ -970,26 +919,18 @@ def test_fits_header(aia171_test_map):
 def test_bad_coordframe_repr(generic_map):
     generic_map.meta['CTYPE1'] = "STUART1"
     generic_map.meta['CTYPE2'] = "STUART2"
-    assert 'Unknown' in generic_map.__repr__()
+    with pytest.warns(UserWarning,
+                      match="Could not determine coordinate frame from map metadata"):
+        assert 'Unknown' in generic_map.__repr__()
 
 
-def test_bad_header_final_fallback():
-    # Checks that if a WCS cannot be constructed from the
-    # header, a warning is raised and a simple WCS is created
-    # instead
+def test_non_str_key():
     header = {'cunit1': 'arcsec',
               'cunit2': 'arcsec',
               None: None,  # Cannot parse this into WCS
               }
-    m = sunpy.map.GenericMap(np.zeros((10, 10)), header)
-    with pytest.warns(UserWarning,
-                      match="Unable to treat `.meta` as a FITS header, assuming a simple WCS."):
-        m.wcs
-    # Simple WCS validation
-    assert list(m.wcs.wcs.ctype) == ['HPLN-', 'HPLT-']
-    assert (m.wcs.wcs.crval == [0.0, 0.0]).all()
-    assert (m.wcs.wcs.crpix == [5.5, 5.5]).all()
-    assert (m.wcs.wcs.cdelt == [1.0, 1.0]).all()
+    with pytest.raises(ValueError, match='All MetaDict keys must be strings'):
+        sunpy.map.GenericMap(np.zeros((10, 10)), header)
 
 
 def test_wcs_isot(aia171_test_map):
@@ -1057,46 +998,6 @@ width_deg = 20 * u.arcsec
 height_deg = 20 * u.arcsec
 
 
-def test_deprecated_submap_inputs(generic_map2, coords):
-    bl_coord, tr_coord, bl_tr_coord = coords
-    # deprecated
-    with pytest.warns(SunpyDeprecationWarning):
-        smap = generic_map2.submap(bl_coord, tr_coord)
-    assert u.allclose(smap.dimensions, (3, 3) * u.pix)
-
-    with pytest.warns(SunpyDeprecationWarning):
-        smap = generic_map2.submap(bl_pix, tr_pix)
-    assert u.allclose(smap.dimensions, (3, 3) * u.pix)
-
-    # error
-    with pytest.raises(TypeError, match="width must be specified as a keyword argument"):
-        generic_map2.submap(bl_coord, width_deg, height_deg)
-
-    with pytest.raises(TypeError, match="width must be specified as a keyword argument"):
-        generic_map2.submap(bl_pix, width_pix, height_pix)
-
-    with pytest.warns(SunpyDeprecationWarning):
-        with pytest.raises(ValueError,
-                           match="Either top_right alone or both width and height must be specified."):
-            generic_map2.submap(bl_coord, width_deg, height=height_deg)
-
-    with pytest.warns(SunpyDeprecationWarning):
-        with pytest.raises(ValueError,
-                           match="Either top_right alone or both width and height must be specified"):
-            generic_map2.submap(bl_pix, width_pix, height=height_pix)
-
-    with pytest.warns(SunpyDeprecationWarning):
-        with pytest.raises(TypeError,
-                           match="Invalid input, top_right must be of type SkyCoord or BaseCoordinateFrame."):
-            generic_map2.submap(bl_coord, width_deg)
-
-    with pytest.warns(SunpyDeprecationWarning):
-        with pytest.raises(ValueError,
-                           match="Either top_right alone or both width and height must be specified"):
-            generic_map2.submap(bl_pix, width_pix, height=height_pix)
-
-
-@pytest.mark.skip
 def test_submap_kwarg_only_input_errors(generic_map2, coords):
     """
     This test replaces the one above when the deprecation period is over.
@@ -1116,42 +1017,6 @@ def test_submap_kwarg_only_input_errors(generic_map2, coords):
             generic_map2.submap(*args, **kwargs)
 
 
-def test_submap_kwarg_only_input_errors(generic_map2, coords):
-    bl_coord, tr_coord, bl_tr_coord = coords
-
-    inputs = (
-        dict(width=width_pix),
-        dict(top_right=tr_pix, width=width_pix),
-        dict(top_right=tr_pix, height=height_pix),
-        dict(),  # Only post deprecation
-    )
-    for kwargs in inputs:
-        with pytest.raises(ValueError, match="top_right alone or both width and height "
-                                             "must be specified"):
-            generic_map2.submap(bl_pix, **kwargs)
-
-    with pytest.raises(TypeError, match="width and height must be a Quantity in units of pixels"):
-        generic_map2.submap(bl_pix, width=width_deg, height=height_deg)
-
-    with pytest.raises(TypeError,
-                       match="top_right must be a Quantity in units of pixels."):
-        generic_map2.submap([10, 10]*u.deg, top_right=[10, 10]*u.deg)
-
-    with pytest.raises(ValueError, match=r"must have shape \(2\, \)"):
-        generic_map2.submap(10*u.pix, top_right=10*u.pix)
-
-    with pytest.raises(ValueError, match=r"must have shape \(2\, \)"):
-        generic_map2.submap([10, 10, 10]*u.pix, top_right=[10, 10, 10]*u.pix)
-
-    with pytest.raises(u.UnitsError):
-        generic_map2.submap([10, 10]*u.deg, width=10*u.km, height=10*u.J)
-
-    with pytest.raises(ValueError,
-                       match="either bottom_left and top_right or bottom_left and height and width should be provided"):
-        generic_map2.submap(SkyCoord([10, 10, 10]*u.deg, [10, 10, 10]*u.deg,
-                                     frame=generic_map2.coordinate_frame))
-
-
 def test_submap_inputs(generic_map2, coords):
     bl_coord, tr_coord, bl_tr_coord = coords
 
@@ -1166,3 +1031,43 @@ def test_submap_inputs(generic_map2, coords):
     for args, kwargs in inputs:
         smap = generic_map2.submap(*args, **kwargs)
         assert u.allclose(smap.dimensions, (3, 3) * u.pix)
+
+
+def test_contour(simple_map):
+    data = np.ones((3, 3))
+    data[1, 1] = 2
+    simple_map = sunpy.map.Map(data, simple_map.meta)
+    # 2 is the central pixel of the map, so contour half way between 1 and 2
+    contours = simple_map.contour(1.5)
+    assert len(contours) == 1
+    contour = contours[0]
+    assert contour.observer.lat == simple_map.observer_coordinate.frame.lat
+    assert contour.observer.lon == simple_map.observer_coordinate.frame.lon
+    assert contour.obstime == simple_map.date
+    assert u.allclose(contour.Tx, [0, -1, 0, 1, 0] * u.arcsec, atol=1e-10 * u.arcsec)
+    assert u.allclose(contour.Ty, [0.5, 0, -0.5, 0, 0.5] * u.arcsec, atol=1e-10 * u.arcsec)
+
+
+def test_contour_units(simple_map):
+    # Check that contouring with units works as intended
+    simple_map.meta['bunit'] = 'm'
+    contours = simple_map.contour(1.5 * u.m)
+    assert len(contours) == 1
+
+    contours_cm = simple_map.contour(150 * u.cm)
+    for c1, c2 in zip(contours, contours_cm):
+        np.all(c1 == c2)
+
+    with pytest.raises(u.UnitsError, match='level must be an astropy quantity convertible to m'):
+        simple_map.contour(1.5)
+    with pytest.raises(u.UnitsError, match='level must be an astropy quantity convertible to m'):
+        simple_map.contour(1.5 * u.s)
+
+
+def test_print_map(generic_map):
+    out_repr = generic_map.__repr__()
+    assert isinstance(out_repr, str)
+    assert object.__repr__(generic_map) in out_repr
+    out_str = generic_map.__str__()
+    assert isinstance(out_str, str)
+    assert out_str in out_repr
