@@ -3,6 +3,7 @@ Test Generic Map
 """
 import os
 
+import matplotlib.colors as mcolor
 import matplotlib.pyplot as plt
 import numpy as np
 import pytest
@@ -14,8 +15,7 @@ import sunpy
 import sunpy.coordinates
 import sunpy.data.test
 import sunpy.map
-from sunpy.coordinates.utils import get_rectangle_coordinates
-from sunpy.tests.helpers import figure_test
+from sunpy.tests.helpers import figure_test, fix_map_wcs
 from sunpy.util.exceptions import SunpyUserWarning
 
 testpath = sunpy.data.test.rootdir
@@ -29,7 +29,8 @@ def aia171_test_map():
 
 @pytest.fixture
 def heliographic_test_map():
-    return sunpy.map.Map(os.path.join(testpath, 'heliographic_phase_map.fits.gz'))
+    m = sunpy.map.Map(os.path.join(testpath, 'heliographic_phase_map.fits.gz'))
+    return fix_map_wcs(m)
 
 
 @pytest.fixture
@@ -121,10 +122,20 @@ def test_rectangle_aia171_top_right(aia171_test_map):
     aia171_test_map.plot()
     bottom_left = SkyCoord(
         0 * u.arcsec, 0 * u.arcsec, frame=aia171_test_map.coordinate_frame)
-    w = 100 * u.arcsec
-    h = 100 * u.arcsec
-    bottom_left, top_right = get_rectangle_coordinates(bottom_left, width=w, height=h)
-    aia171_test_map.draw_rectangle(bottom_left, top_right=top_right)
+    top_right = SkyCoord(
+        100 * u.arcsec, 100 * u.arcsec, frame=aia171_test_map.coordinate_frame)
+    aia171_test_map.draw_rectangle(bottom_left, top_right=top_right, label='Rectangle')
+    plt.legend()  # Check that the 'Rectangle' label shows up in the legend
+
+
+@figure_test
+def test_rectangle_aia171_hgs_width_height(aia171_test_map):
+    aia171_test_map.plot()
+    bottom_left = SkyCoord(
+        0 * u.deg, -60 * u.deg, frame='heliographic_stonyhurst', obstime=aia171_test_map.date)
+    w = 45 * u.deg
+    h = 120 * u.deg
+    aia171_test_map.draw_rectangle(bottom_left, width=w, height=h)
 
 
 @figure_test
@@ -184,7 +195,7 @@ def test_heliographic_rectangle_width_height(heliographic_test_map):
         60 * u.deg, 50 * u.deg, frame=heliographic_test_map.coordinate_frame)
     w = 13 * u.deg
     h = 13 * u.deg
-    heliographic_test_map.draw_rectangle(bottom_left, width=w, height=h, color='cyan')
+    heliographic_test_map.draw_rectangle(bottom_left, width=w, height=h, edgecolor='cyan')
 
 
 @figure_test
@@ -192,13 +203,23 @@ def test_heliographic_rectangle_top_right(heliographic_test_map):
     heliographic_test_map.plot()
     bottom_left = SkyCoord(
         60 * u.deg, 50 * u.deg, frame=heliographic_test_map.coordinate_frame)
-    w = 13 * u.deg
-    h = 13 * u.deg
-    bottom_left, top_right = get_rectangle_coordinates(bottom_left, width=w, height=h)
-    heliographic_test_map.draw_rectangle(bottom_left, width=w, height=h, color='cyan')
+    top_right = SkyCoord(73 * u.deg, 63 * u.deg, frame=heliographic_test_map.coordinate_frame)
+    heliographic_test_map.draw_rectangle(bottom_left, top_right=top_right, edgecolor='cyan')
 
 
+# See https://github.com/sunpy/sunpy/issues/4294 to track this warning. Ideally
+# it should not be filtered, and the cause of it fixed.
+@pytest.mark.filterwarnings(r'ignore:Numpy has detected that you \(may be\) writing to an array with\noverlapping memory')
 @figure_test
 def test_heliographic_grid_annotations(heliographic_test_map):
     heliographic_test_map.plot()
     heliographic_test_map.draw_grid(annotate=False)
+
+
+def test_plot_norm_error(aia171_test_map):
+    # Check that duplicating vmin, vmax raises an error
+    norm = mcolor.Normalize(vmin=0, vmax=1)
+    with pytest.raises(ValueError, match='Cannot manually specify vmin'):
+        aia171_test_map.plot(norm=norm, vmin=0)
+    with pytest.raises(ValueError, match='Cannot manually specify vmax'):
+        aia171_test_map.plot(norm=norm, vmax=0)
