@@ -10,16 +10,20 @@ from sunpy.net.dataretriever.client import QueryResponse
 from sunpy.net.dataretriever.sources import noaa
 from sunpy.tests.helpers import no_vso
 from sunpy.time import parse_time
-from sunpy.time.timerange import TimeRange
 
 
 @pytest.fixture
-def LCClient():
+def indices_client():
     return noaa.NOAAIndicesClient()
 
 
 @pytest.fixture
-def SRSClient():
+def predict_client():
+    return noaa.NOAAPredictClient()
+
+
+@pytest.fixture
+def srs_client():
     return noaa.SRSClient()
 
 
@@ -28,66 +32,62 @@ def mock_query_object(start_date, end_date):
     Creation of a QueryResponse object, and prefill some
     downloaded data from noaa.NOAAIndicesClient().fetch(Time('20 ..)
     """
-    # Create a mock QueryResponse object
-    map_ = {
-        'TimeRange': TimeRange(parse_time(start_date), parse_time(end_date)),
-        'Time_start': parse_time(start_date),
-        'Time_end': parse_time(end_date),
-        'source': 'sdic',
-        'instrument': 'noaa-indices',
-        'physobs': 'sunspot number',
-        'provider': 'swpc'
+    # Create a mock Query Response object
+    start = parse_time(start_date)
+    end = parse_time(end_date)
+    obj = {
+        'Start Time': parse_time(start),
+        'End Time': parse_time(end),
+        'Instrument': 'NOAA-Indices',
+        'Physobs': 'sunspot number',
+        'Source': 'SIDC',
+        'Provider': 'SWPC',
+        'url': 'https://services.swpc.noaa.gov/json/solar-cycle/observed-solar-cycle-indices.json'
     }
-
-    resp = QueryResponse.create(map_,
-                                noaa.NOAAIndicesClient()._get_url_for_timerange(None),
-                                client=noaa.NOAAIndicesClient())
-    return resp
+    results = QueryResponse([obj], client=noaa.NOAAIndicesClient())
+    return results
 
 
 @pytest.mark.remote_data
-def test_fetch_working(LCClient, tmpdir):
+def test_fetch_working(indices_client, tmpdir):
     """
     Tests if the online server for noaa is working.
     Uses the url : https://services.swpc.noaa.gov/json/solar-cycle/observed-solar-cycle-indices.json
     """
-    qr1 = LCClient.search(Time('2012/10/4', '2012/10/6'),
-                          Instrument('noaa-indices'))
+    qr1 = indices_client.search(Time('2012/10/4', '2012/10/6'),
+                                Instrument('noaa-indices'))
 
     # Mock QueryResponse object
     mock_qr = mock_query_object('2012/10/4', '2012/10/6')
 
     # Compare if two objects have the same attribute
 
-    mock_qr = mock_qr.blocks[0]
-    qr = qr1.blocks[0]
+    mock_qr = mock_qr[0]
+    qr = qr1[0]
 
-    assert mock_qr.source == qr.source
-    assert mock_qr.provider == qr.provider
-    assert mock_qr.physobs == qr.physobs
-    assert mock_qr.instrument == qr.instrument
-    assert mock_qr.url == qr.url
-    assert mock_qr.time == qr.time
-
-    # Assert if the timerange is same
-    assert qr1.time_range() == TimeRange('2012/10/4', '2012/10/6')
+    assert mock_qr['Source'] == qr['Source']
+    assert mock_qr['Provider'] == qr['Provider']
+    assert mock_qr['Physobs'] == qr['Physobs']
+    assert mock_qr['Instrument'] == qr['Instrument']
+    assert mock_qr['url'] == qr['url']
 
     target_dir = tmpdir.mkdir("down")
-    download_list = LCClient.fetch(qr1, path=target_dir)
+    download_list = indices_client.fetch(qr1, path=target_dir)
     assert len(download_list) == len(qr1)
     assert download_list[0].split('/')[-1] == 'observed-solar-cycle-indices.json'
 
 
 @pytest.mark.parametrize(
     "timerange,url_start,url_end",
-    [(TimeRange('1995/06/03', '1995/06/04'),
+    [(Time('1995/06/03', '1995/06/04'),
       'https://services.swpc.noaa.gov/json/solar-cycle/observed-solar-cycle-indices.json',
       'https://services.swpc.noaa.gov/json/solar-cycle/observed-solar-cycle-indices.json'),
-     (TimeRange('2008/06/01', '2008/06/02'),
+     (Time('2008/06/01', '2008/06/02'),
       'https://services.swpc.noaa.gov/json/solar-cycle/observed-solar-cycle-indices.json',
       'https://services.swpc.noaa.gov/json/solar-cycle/observed-solar-cycle-indices.json')])
-def test_get_url_for_time_range(LCClient, timerange, url_start, url_end):
-    urls = LCClient._get_url_for_timerange(timerange)
+def test_get_url_for_time_range(indices_client, timerange, url_start, url_end):
+    resp = indices_client.search(timerange)
+    urls = [i['url'] for i in resp]
     assert isinstance(urls, list)
     assert urls[0] == url_start
     assert urls[-1] == url_end
@@ -107,8 +107,8 @@ def test_can_handle_query():
 
 @mock.patch('sunpy.net.dataretriever.sources.noaa.NOAAIndicesClient.search',
             return_value=mock_query_object('2012/8/9', '2012/8/10'))
-def test_query(mock_search, LCClient):
-    qr1 = LCClient.search(
+def test_query(mock_search, indices_client):
+    qr1 = indices_client.search(
         Time('2012/8/9', '2012/8/10'), Instrument('noaa-indices'))
     assert isinstance(qr1, QueryResponse)
     assert len(qr1) == 1
@@ -122,12 +122,12 @@ def test_query(mock_search, LCClient):
 @mock.patch('parfive.Downloader.download',
             return_value=None)
 @mock.patch('parfive.Downloader.enqueue_file')
-def test_fetch(mock_wait, mock_search, mock_enqueue, tmp_path, LCClient):
+def test_fetch(mock_wait, mock_search, mock_enqueue, tmp_path, indices_client):
     path = tmp_path / "sub"
     path.mkdir()
-    qr1 = LCClient.search(Time('2012/10/4', '2012/10/6'),
-                          Instrument('noaa-indices'))
-    LCClient.fetch(qr1, path=path / "{file}")
+    qr1 = indices_client.search(Time('2012/10/4', '2012/10/6'),
+                                Instrument('noaa-indices'))
+    indices_client.fetch(qr1, path=path / "{file}")
 
     # Here we assert that the `fetch` function has called the parfive
     # Downloader.enqueue_file method with the correct arguments. Everything
@@ -137,14 +137,14 @@ def test_fetch(mock_wait, mock_search, mock_enqueue, tmp_path, LCClient):
                                           path / "observed-solar-cycle-indices.json"))
 
 
-@no_vso
 @mock.patch('sunpy.net.dataretriever.sources.noaa.NOAAIndicesClient.search',
             return_value=mock_query_object('2012/10/4', '2012/10/6'))
 # The return value of download is irrelevant
 @mock.patch('parfive.Downloader.download',
             return_value=None)
 @mock.patch('parfive.Downloader.enqueue_file')
-def test_fido(mock_wait, mock_search, mock_enqueue, tmp_path, LCClient):
+@no_vso
+def test_fido(mock_wait, mock_search, mock_enqueue, tmp_path, indices_client):
     path = tmp_path / "sub"
     path.mkdir()
     qr1 = Fido.search(Time('2012/10/4', '2012/10/6'),
@@ -159,6 +159,7 @@ def test_fido(mock_wait, mock_search, mock_enqueue, tmp_path, LCClient):
                                           path / "observed-solar-cycle-indices.json"))
 
 
+@no_vso
 @pytest.mark.remote_data
 def test_srs_tar_unpack():
     qr = Fido.search(a.Instrument("soon") & a.Time("2015/01/01", "2015/01/01T23:59:29"))
@@ -167,6 +168,7 @@ def test_srs_tar_unpack():
     assert res.data[0].endswith("20150101SRS.txt")
 
 
+@no_vso
 @pytest.mark.remote_data
 def test_srs_tar_unpack_midyear():
     qr = Fido.search(a.Instrument("soon") & a.Time("2011/06/07", "2011/06/08T23:59:29"))
@@ -176,15 +178,27 @@ def test_srs_tar_unpack_midyear():
     assert res.data[-1].endswith("20110608SRS.txt")
 
 
+@no_vso
+@pytest.mark.remote_data
+@mock.patch('ftplib.FTP.nlst', side_effect=[[''], ['20200101SRS.txt', '20200102SRS.txt']])
+def test_srs_missing_tarball(mock_ftp_nlst):
+    qr = Fido.search(a.Time('2020-01-01', '2020-01-02'), a.Instrument.srs_table)
+    urls = [qrblock['url'] for qrblock in qr[0]]
+    assert urls[0].endswith('20200101SRS.txt')
+    assert urls[1].endswith('20200102SRS.txt')
+
+
+@no_vso
 @pytest.mark.remote_data
 def test_srs_current_year():
     year = datetime.date.today().year
-    qr = Fido.search(a.Instrument("soon") & a.Time(f"{year}/01/01", f"{year}/01/01T23:59:29"))
+    qr = Fido.search(a.Instrument("soon") & a.Time(f"{year}/02/01", f"{year}/02/01T23:59:29"))
     res = Fido.fetch(qr)
     assert len(res) == 1
-    assert res.data[0].endswith(f"{year}0101SRS.txt")
+    assert res.data[0].endswith(f"{year}0201SRS.txt")
 
 
+@no_vso
 @pytest.mark.remote_data
 def test_srs_save_path(tmpdir):
     qr = Fido.search(a.Instrument.srs_table, a.Time("2016/10/01", "2016/10/02"))
@@ -194,14 +208,33 @@ def test_srs_save_path(tmpdir):
     assert files[1].endswith("20161002SRS.txt")
 
 
+@pytest.mark.remote_data
 @pytest.mark.filterwarnings('ignore:ERFA function')
-def test_srs_out_of_range(SRSClient):
-    res = SRSClient.search(a.Time('1995/01/01', '1995/02/01'))
+def test_srs_out_of_range(srs_client):
+    res = srs_client.search(a.Time('1995/01/01', '1995/02/01'))
     assert len(res) == 0
-    res = SRSClient.search(a.Time('2995/01/01', '2995/02/01'))
+    res = srs_client.search(a.Time('2995/01/01', '2995/02/01'))
     assert len(res) == 0
 
 
+@pytest.mark.remote_data
+@pytest.mark.filterwarnings('ignore:ERFA function')
+def test_srs_start_or_end_out_of_range(srs_client):
+    res = srs_client.search(a.Time('1995/12/30', '1996/01/02'))
+    assert len(res) == 1
+    cur_year = datetime.date.today().year
+    res = srs_client.search(a.Time(f'{cur_year}/01/01', f'{cur_year+2}/01/01'))
+    assert len(res) > 0
+
+
+def test_no_time(predict_client, indices_client):
+    res = indices_client.search(a.Instrument.noaa_indices)
+    assert len(res) == 1
+
+    res = predict_client.search(a.Instrument.noaa_predict)
+    assert len(res) == 1
+
+
 def test_attr_reg():
     assert a.Instrument.noaa_indices == a.Instrument("NOAA-Indices")
     assert a.Instrument.noaa_predict == a.Instrument("NOAA-Predict")
@@ -209,9 +242,19 @@ def test_attr_reg():
     assert a.Instrument.soon == a.Instrument("SOON")
 
 
-def test_client_repr(LCClient):
+def test_client_repr(indices_client):
     """
     Repr check
     """
-    output = str(LCClient)
+    output = str(indices_client)
     assert output[:50] == 'sunpy.net.dataretriever.sources.noaa.NOAAIndicesCl'
+
+
+def test_show():
+    mock_qr = mock_query_object('2012/10/4', '2012/10/6')
+    qrshow0 = mock_qr.show()
+    qrshow1 = mock_qr.show('Source', 'Instrument')
+    allcols = {'Start Time', 'End Time', 'Instrument', 'Physobs', 'Source', 'Provider', 'url'}
+    assert not allcols.difference(qrshow0.colnames)
+    assert qrshow1.colnames == ['Source', 'Instrument']
+    assert qrshow0['Instrument'][0] == 'NOAA-Indices'
