@@ -17,8 +17,10 @@ from astropy.utils.misc import isiterable
 
 from sunpy import config
 from sunpy.net.attr import and_
-from sunpy.net.base_client import BaseClient, BaseQueryResponse
+from sunpy.net.base_client import BaseClient, QueryResponseTable, convert_row_to_table
 from sunpy.net.jsoc.attrs import walker
+from sunpy.util._table_attribute import TableAttribute
+from sunpy.util.decorators import deprecated
 from sunpy.util.exceptions import SunpyUserWarning
 from sunpy.util.parfive_helpers import Downloader, Results
 
@@ -33,71 +35,32 @@ class NotExportedError(Exception):
     pass
 
 
-class JSOCResponse(BaseQueryResponse):
-    def __init__(self, table=None, client=None):
-        """
-        table : `astropy.table.Table`
-        """
-        super().__init__()
-        self.table = table or astropy.table.QTable()
-        self.query_args = getattr(table, 'query_args', None)
-        self.requests = getattr(table, 'requests', None)
-        self._client = client
+class JSOCResponse(QueryResponseTable):
+    query_args = TableAttribute()
+    requests = TableAttribute()
+    display_keys = ['T_REC', 'TELESCOP', 'INSTRUME', 'WAVELNTH', 'CAR_ROT']
+    # This variable is used to detect if the result has been sliced before it is passed
+    # to fetch and issue a warning to the user about not being able to post-filter JSOC searches.
+    _original_num_rows = TableAttribute(default=None)
 
-    @property
-    def client(self):
-        return self._client
-
-    @client.setter
-    def client(self, client):
-        self._client = client
-
-    @property
-    def blocks(self):
-        return list(self.table.iterrows)
-
-    def __len__(self):
-        if self.table is None:
-            return 0
-        else:
-            return len(self.table)
-
-    def __getitem__(self, item):
-        if isinstance(item, int):
-            item = slice(item, item + 1)
-        ret = type(self)(self.table[item])
-        ret.query_args = self.query_args
-        ret.requests = self.requests
-        ret.client = self._client
-
-        warnings.warn("Downloading of sliced JSOC results is not supported. "
-                      "All the files present in the original response will be downloaded.",
-                      SunpyUserWarning)
-        return ret
-
-    def __iter__(self):
-        return (t for t in [self])
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._original_num_rows = len(self)
 
+    # TODO: remove this method post 3.0
     def build_table(self):
-        return self.table
+        # remove this check post 3.0
+        if self.query_args is not None and any('keys' in i for i in self.query_args):
+            new_table = self.copy()
+            new_table.display_keys = slice(None)
+            return new_table
 
-    def append(self, table):
-        if self.table is None:
-            self.table = table
-        else:
-            self.table = astropy.table.vstack([self.table, table])
-
-    def response_block_properties(self):
-        """
-        Returns a set of class attributes on all the response blocks.
-        """
-        warnings.warn("The JSOC client does not support response block properties.", SunpyUserWarning)
-        return set()
+        return self
 
 
 class JSOCClient(BaseClient):
     """
-    This is a client to the JSOC Data Export service.
+    Provides access to the JSOC Data Export service.
 
     It exposes a similar API to the VSO client, although the underlying model
     is more complex. The JSOC stages data before you can download it, so a JSOC
@@ -141,7 +104,7 @@ class JSOCClient(BaseClient):
         The response object holds the records that your query will return:
 
         >>> print(response)   # doctest: +REMOTE_DATA
-                T_REC          TELESCOP  INSTRUME  WAVELNTH CAR_ROT
+                 T_REC          TELESCOP  INSTRUME  WAVELNTH CAR_ROT
         ----------------------- -------- ---------- -------- -------
         2014.01.01_00:00:45_TAI  SDO/HMI HMI_FRONT2   6173.0    2145
         2014.01.01_00:01:30_TAI  SDO/HMI HMI_FRONT2   6173.0    2145
@@ -195,7 +158,7 @@ class JSOCClient(BaseClient):
         >>> client = jsoc.JSOCClient()  # doctest: +REMOTE_DATA
         >>> response = client.search(a.Time('2014/1/1T00:00:00', '2014/1/1T00:00:36'),
         ...                          a.jsoc.Series('aia.lev1_euv_12s'), a.jsoc.Segment('image'),
-        ...                          a.jsoc.Wavelength(171*u.AA), a.jsoc.Notify("sunpy@sunpy.org"))  # doctest: +REMOTE_DATA
+        ...                          a.Wavelength(171*u.AA), a.jsoc.Notify("sunpy@sunpy.org"))  # doctest: +REMOTE_DATA
 
         The response object holds the records that your query will return:
 
@@ -245,10 +208,10 @@ class JSOCClient(BaseClient):
         Build a JSOC query and submit it to JSOC for processing.
 
         Takes a variable number of `~sunpy.net.jsoc.attrs` as parameters,
-        which are chained together using the AND (`&`) operator.
+        which are chained together using the AND (``&``) operator.
 
         Complex queries to be easily formed using logical operators such as
-        `&` and `|`, in the same way as the VSO client.
+        ``&`` and ``|``, in the same way as the VSO client.
 
         Parameters
         ----------
@@ -274,7 +237,7 @@ class JSOCClient(BaseClient):
             >>> from sunpy.net import attrs as a
             >>> client = jsoc.JSOCClient()  # doctest: +REMOTE_DATA
             >>> response = client.search(a.Time('2017-09-06T12:00:00', '2017-09-06T12:02:00'),
-            ...                          a.jsoc.Series('aia.lev1_euv_12s'), a.jsoc.Wavelength(304*u.AA),
+            ...                          a.jsoc.Series('aia.lev1_euv_12s'), a.Wavelength(304*u.AA),
             ...                          a.jsoc.Segment('image'))  # doctest: +REMOTE_DATA
             >>> print(response)  # doctest: +REMOTE_DATA
                    T_REC         TELESCOP INSTRUME WAVELNTH CAR_ROT
@@ -293,34 +256,33 @@ class JSOCClient(BaseClient):
 
         *Example 2*
 
-        Request keyword data of ``hmi.v_45s`` for certain specific keywords only::
+        Request keyword data of ``hmi.v_45s`` and show specific columns only::
 
             >>> import astropy.units as u
             >>> from sunpy.net import jsoc
             >>> from sunpy.net import attrs as a
             >>> client = jsoc.JSOCClient()  # doctest: +REMOTE_DATA
             >>> response = client.search(a.Time('2014-01-01T00:00:00', '2014-01-01T00:10:00'),
-            ...                          a.jsoc.Series('hmi.v_45s'),
-            ...                          a.jsoc.Keys('T_REC, DATAMEAN, OBS_VR'))  # doctest: +REMOTE_DATA
-            >>> print(response)  # doctest: +REMOTE_DATA
-                         T_REC               DATAMEAN            OBS_VR
-            ----------------------- ------------------ ------------------
-            2014.01.01_00:00:45_TAI        1906.518188        1911.202614
-            2014.01.01_00:01:30_TAI        1908.876221        1913.945512
-            2014.01.01_00:02:15_TAI          1911.7771 1916.6679989999998
-            2014.01.01_00:03:00_TAI        1913.422485 1919.3699239999999
-            2014.01.01_00:03:45_TAI        1916.500488        1922.050862
-            2014.01.01_00:04:30_TAI        1920.414795 1924.7110050000001
-            2014.01.01_00:05:15_TAI        1922.636963         1927.35015
-            2014.01.01_00:06:00_TAI 1924.6973879999998        1929.968523
-            2014.01.01_00:06:45_TAI        1927.758301 1932.5664510000001
-            2014.01.01_00:07:30_TAI        1929.646118         1935.14288
-            2014.01.01_00:08:15_TAI        1932.097046        1937.698521
-            2014.01.01_00:09:00_TAI 1935.7286379999998         1940.23353
-            2014.01.01_00:09:45_TAI        1937.754028        1942.747605
-            2014.01.01_00:10:30_TAI 1940.1462399999998        1945.241147
+            ...                          a.jsoc.Series('hmi.v_45s'))  # doctest: +REMOTE_DATA
+            >>> print(response.show('T_REC', 'WAVELNTH', 'CAR_ROT'))  # doctest: +REMOTE_DATA
+                     T_REC          WAVELNTH CAR_ROT
+            ----------------------- -------- -------
+            2014.01.01_00:00:45_TAI   6173.0    2145
+            2014.01.01_00:01:30_TAI   6173.0    2145
+            2014.01.01_00:02:15_TAI   6173.0    2145
+            2014.01.01_00:03:00_TAI   6173.0    2145
+            2014.01.01_00:03:45_TAI   6173.0    2145
+            2014.01.01_00:04:30_TAI   6173.0    2145
+            2014.01.01_00:05:15_TAI   6173.0    2145
+            2014.01.01_00:06:00_TAI   6173.0    2145
+            2014.01.01_00:06:45_TAI   6173.0    2145
+            2014.01.01_00:07:30_TAI   6173.0    2145
+            2014.01.01_00:08:15_TAI   6173.0    2145
+            2014.01.01_00:09:00_TAI   6173.0    2145
+            2014.01.01_00:09:45_TAI   6173.0    2145
+            2014.01.01_00:10:30_TAI   6173.0    2145
 
-            *Example 3*
+        *Example 3*
 
         Request data of ``aia.lev1_euv_12s`` on the basis of PrimeKeys other than ``T_REC``::
 
@@ -351,10 +313,12 @@ class JSOCClient(BaseClient):
             iargs.update(block)
             # Update blocks with deep copy of iargs because in _make_recordset we use .pop() on element from iargs
             blocks.append(copy.deepcopy(iargs))
-            return_results.append(self._lookup_records(iargs))
+            return_results = astropy.table.vstack([return_results, self._lookup_records(iargs)])
         return_results.query_args = blocks
+        return_results._original_num_rows = len(return_results)
         return return_results
 
+    @deprecated(since="2.1", message="use JSOCClient.search() instead", alternative="JSOCClient.search()")
     def search_metadata(self, *query, **kwargs):
         """
         Get the metadata of all the files obtained in a search query.
@@ -374,31 +338,6 @@ class JSOCClient(BaseClient):
         res : `~pandas.DataFrame` object
             A collection of metadata of all the files.
 
-        Example
-        -------
-
-        Request metadata or all all AIA 304 image data between 2014-01-01T00:00 and
-        2014-01-01T01:00.
-
-        Since, the function only performs a lookdata, and does not make a proper export
-        request, attributes like Segment need not be passed::
-
-            >>> import astropy.units as u
-            >>> from sunpy.net import jsoc
-            >>> from sunpy.net import attrs as a
-            >>> client = jsoc.JSOCClient()  # doctest: +REMOTE_DATA
-            >>> metadata = client.search_metadata(
-            ...                         a.Time('2014-01-01T00:00:00', '2014-01-01T00:01:00'),
-            ...                         a.jsoc.Series('aia.lev1_euv_12s'), a.jsoc.Wavelength(304*u.AA))  # doctest: +REMOTE_DATA
-            >>> print(metadata[['T_OBS', 'WAVELNTH']])  # doctest: +REMOTE_DATA
-                                                                        T_OBS  WAVELNTH
-            aia.lev1_euv_12s[2014-01-01T00:00:01Z][304]  2014-01-01T00:00:08.57Z       304
-            aia.lev1_euv_12s[2014-01-01T00:00:13Z][304]  2014-01-01T00:00:20.58Z       304
-            aia.lev1_euv_12s[2014-01-01T00:00:25Z][304]  2014-01-01T00:00:32.57Z       304
-            aia.lev1_euv_12s[2014-01-01T00:00:37Z][304]  2014-01-01T00:00:44.58Z       304
-            aia.lev1_euv_12s[2014-01-01T00:00:49Z][304]  2014-01-01T00:00:56.57Z       304
-            aia.lev1_euv_12s[2014-01-01T00:01:01Z][304]  2014-01-01T00:01:08.59Z       304
-
         """
         query = and_(*query)
         blocks = []
@@ -409,10 +348,9 @@ class JSOCClient(BaseClient):
             iargs.update({'meta': True})
             blocks.append(iargs)
             res = res.append(self._lookup_records(iargs))
-
         return res
 
-    def request_data(self, jsoc_response, **kwargs):
+    def request_data(self, jsoc_response, method='url', **kwargs):
         """
         Request that JSOC stages the data for download. This method will not
         wait for the request to be staged.
@@ -422,10 +360,17 @@ class JSOCClient(BaseClient):
         jsoc_response : `~sunpy.net.jsoc.jsoc.JSOCResponse` object
             The results of a query
 
+        method : {'url', 'url-tar', 'url-quick'}
+            Method for requesting JSOC data, can be 'url-tar', 'url' (the default) and 'url-quick'
+            If 'url-tar' it will request JSOC to provide single .tar file which contains all data
+            If 'url' it will request JSOC to provide all data as separate .fits files
+            If 'url-quick' (only with protocol 'as-is') provide all data as separate files,
+            but only if data is online.
+
         Returns
         -------
-        requests : `~drms.ExportRequest` object or
-                   a list of  `~drms.ExportRequest` objects
+        requests : `~drms.client.ExportRequest` object or
+                   a list of  `~drms.client.ExportRequest` objects
 
             Request Id can be accessed by requests.id
             Request status can be accessed by requests.status
@@ -434,19 +379,28 @@ class JSOCClient(BaseClient):
 
         requests = []
         self.query_args = jsoc_response.query_args
+        supported_protocols = {'fits', 'as-is'}
+        supported_methods = {'url-tar', 'url', 'url-quick'}
         for block in jsoc_response.query_args:
 
             ds = self._make_recordset(**block)
             cd = drms.Client(email=block.get('notify', ''))
             protocol = block.get('protocol', 'fits')
+            cutout = block.get('cutout')
 
-            if protocol != 'fits' and protocol != 'as-is':
-                error_message = "Protocols other than fits and as-is are "\
+            if protocol not in supported_protocols:
+                error_message = f"Protocols other than {','.join(supported_protocols)} "\
                                 "are not supported."
                 raise TypeError(error_message)
+            if method not in supported_methods:
+                error_message = f"Methods other than {','.join(supported_methods)} "\
+                                "are not supported."
+                raise TypeError(error_message)
+            process = {'im_patch': cutout} if cutout is not None else None
 
-            method = 'url' if protocol == 'fits' else 'url_quick'
-            r = cd.export(ds, method=method, protocol=protocol)
+            if method != 'url-tar':
+                method = 'url' if protocol == 'fits' else 'url_quick'
+            r = cd.export(ds, method=method, protocol=protocol, process=process)
 
             requests.append(r)
 
@@ -454,12 +408,20 @@ class JSOCClient(BaseClient):
             return requests[0]
         return requests
 
+    @convert_row_to_table
     def fetch(self, jsoc_response, path=None, progress=True, overwrite=False,
               downloader=None, wait=True, sleep=10, max_conn=default_max_conn, **kwargs):
         """
         Make the request for the data in a JSOC response and wait for it to be
         staged and then download the data.
 
+        .. note::
+
+            **Only complete searches can be downloaded from JSOC**, this means
+            that no slicing operations performed on the results object will
+            affect the number of files downloaded.
+
+
         Parameters
         ----------
         jsoc_response : `~sunpy.net.jsoc.jsoc.JSOCResponse` object
@@ -476,7 +438,7 @@ class JSOCClient(BaseClient):
             Determine how to handle downloading if a file already exists with the
             same name. If `False` the file download will be skipped and the path
             returned to the existing file, if `True` the file will be downloaded
-            and the existing file will be overwritten, if `'unique'` the filename
+            and the existing file will be overwritten, if ``'unique'`` the filename
             will be modified to be unique.
 
         max_conn : `int`
@@ -487,7 +449,7 @@ class JSOCClient(BaseClient):
 
         wait : `bool`, optional
            If `False` ``downloader.download()`` will not be called. Only has
-           any effect if `downloader` is not `None`.
+           any effect if ``downloader`` is not `None`.
 
         sleep : `int`
             The number of seconds to wait between calls to JSOC to check the status
@@ -498,7 +460,19 @@ class JSOCClient(BaseClient):
         results : a `~sunpy.net.download.Results` instance
             A Results object
 
-        """
+       """
+        for resp in jsoc_response.query_args:
+            if 'notify' not in resp:
+                raise ValueError('A registered email is required to get data from JSOC. '
+                                 'Please supply an email with attrs.jsoc.Notify to Fido.search. '
+                                 'Then pass those new results back into Fido.fetch')
+
+        if len(jsoc_response) != jsoc_response._original_num_rows:
+            warnings.warn("Downloading of sliced JSOC results is not supported. "
+                          "All the files present in the original response will "
+                          "be downloaded when passed to fetch().",
+                          SunpyUserWarning)
+
         # Make staging request to JSOC
         responses = self.request_data(jsoc_response)
 
@@ -529,8 +503,8 @@ class JSOCClient(BaseClient):
 
         Parameters
         ----------
-        requests : `~drms.ExportRequest`, `str`, `list`
-            `~drms.ExportRequest` objects or `str` request IDs or lists
+        requests : `~drms.client.ExportRequest`, `str`, `list`
+            `~drms.client.ExportRequest` objects or `str` request IDs or lists
             returned by `~sunpy.net.jsoc.jsoc.JSOCClient.request_data`.
 
         path : `str`
@@ -544,7 +518,7 @@ class JSOCClient(BaseClient):
             Determine how to handle downloading if a file already exists with the
             same name. If `False` the file download will be skipped and the path
             returned to the existing file, if `True` the file will be downloaded
-            and the existing file will be overwritten, if `'unique'` the filename
+            and the existing file will be overwritten, if ``'unique'`` the filename
             will be modified to be unique.
 
         downloader : `parfive.Downloader`, optional
@@ -591,35 +565,37 @@ class JSOCClient(BaseClient):
 
         paths = []
         for request in requests:
-            for filename in request.data['filename']:
-                # Ensure we don't duplicate the file extension
-                ext = os.path.splitext(filename)[1]
-                if path.endswith(ext):
-                    fname = path.strip(ext)
-                else:
-                    fname = path
-                fname = fname.format(file=filename)
-                fname = os.path.expanduser(fname)
-                paths.append(fname)
-
-        if max_conn * kwargs['max_splits'] > 10:
-            warnings.warn(("JSOC does not support more than 10 parallel connections. " +
-                           f"Changing the number of parallel connections to {2 * self.default_max_conn}."),
-                          SunpyUserWarning)
-            kwargs['max_splits'] = 2
-            max_conn = self.default_max_conn
+            if request.method == 'url-tar':
+                fname = path.format(file=Path(request.tarfile).name)
+                paths.append(os.path.expanduser(fname))
+            else:
+                for filename in request.data['filename']:
+                    # Ensure we don't duplicate the file extension
+                    ext = os.path.splitext(filename)[1]
+                    if path.endswith(ext):
+                        fname = path.strip(ext)
+                    else:
+                        fname = path
+                    fname = fname.format(file=filename)
+                    fname = os.path.expanduser(fname)
+                    paths.append(fname)
 
         dl_set = True
         if not downloader:
             dl_set = False
             downloader = Downloader(progress=progress, overwrite=overwrite, max_conn=max_conn)
-        else:
-            downloader.max_conn = max_conn
+
+        if downloader.max_conn * kwargs['max_splits'] > 10:
+            warnings.warn(("JSOC does not support more than 10 parallel connections. " +
+                           f"Changing the number of parallel connections to {2 * self.default_max_conn}."),
+                          SunpyUserWarning)
+            kwargs['max_splits'] = 2
+            downloader.max_conn = self.default_max_conn
 
         urls = []
         for request in requests:
             if request.status == 0:
-                if request.protocol == 'as-is':
+                if request.protocol == 'as-is' or request.method == 'url-tar':
                     urls.extend(list(request.urls.url))
                 else:
                     for index, data in request.data.iterrows():
@@ -807,14 +783,15 @@ class JSOCClient(BaseClient):
         Do a LookData request to JSOC to workout what results the query returns.
         """
 
-        keywords_default = ['T_REC', 'TELESCOP', 'INSTRUME', 'WAVELNTH', 'CAR_ROT']
         isMeta = iargs.get('meta', False)
         c = drms.Client()
 
         if isMeta:
             keywords = '**ALL**'
         else:
-            keywords = iargs.get('keys', keywords_default)
+            keywords = iargs.get('keys', '**ALL**')
+        # TODO: keywords should be set only to '**ALL**' post 3.0
+        # All checks done above should be removed.
 
         if 'series' not in iargs:
             error_message = "Series must be specified for a JSOC Query"
@@ -882,6 +859,7 @@ class JSOCClient(BaseClient):
 
         # If the method was called from search_metadata(), return a Pandas Dataframe,
         # otherwise return astropy.table
+        # TODO: this check should also be removed post 3.0
         if isMeta:
             return r
 
@@ -897,7 +875,8 @@ class JSOCClient(BaseClient):
 
         required = {a.jsoc.Series}
         optional = {a.jsoc.Protocol, a.jsoc.Notify, a.Wavelength, a.Time,
-                    a.jsoc.Segment, a.jsoc.Keys, a.jsoc.PrimeKey, a.Sample}
+                    a.jsoc.Segment, a.jsoc.Keys, a.jsoc.PrimeKey, a.Sample,
+                    a.jsoc.Cutout}
         return cls.check_attr_types_in_query(query, required, optional)
 
     @classmethod
@@ -940,7 +919,7 @@ class JSOCClient(BaseClient):
             keyword_info = {}
             keyword_info["series_store"] = series_store
             keyword_info["segments"] = segments
-            json.dump(keyword_info, attrs_file)
+            json.dump(keyword_info, attrs_file, indent=2)
 
     @staticmethod
     def load_jsoc_values():
