@@ -61,6 +61,7 @@ TIME_FORMAT_LIST = [
     "%Y:%j:%H:%M:%S.%f",  # Example 2012:124:21:08:12.999999
     "%Y%m%d%H%M%S",  # Example 20140101000001 (JSOC / VSO)
     "%Y.%m.%d_%H:%M:%S_TAI",  # Example 2016.05.04_21:08:12_TAI
+    "%Y/%m/%dT%H:%M",  # Example 2007/05/04T21:08
 ]
 
 
@@ -195,6 +196,19 @@ def convert_time_astropy(time_string, **kwargs):
     return time_string
 
 
+@convert_time.register(list)
+def convert_time_list(time_list, format=None, **kwargs):
+    item = time_list[0]
+    # If we have a list of strings, need to get the correct format from our
+    # list of custom formats.
+    if isinstance(item, str) and format is None:
+        string_format = _get_time_fmt(item)
+        return Time.strptime(time_list, string_format, **kwargs)
+
+    # Otherwise return the default method
+    return convert_time.dispatch(object)(time_list, format, **kwargs)
+
+
 @convert_time.register(str)
 def convert_time_str(time_string, **kwargs):
     # remove trailing zeros and the final dot to allow any
@@ -221,6 +235,17 @@ def convert_time_str(time_string, **kwargs):
     return convert_time.dispatch(object)(time_string, **kwargs)
 
 
+def _get_time_fmt(time_string):
+    """
+    Try all the formats in TIME_FORMAT_LIST to work out which one applies to
+    the time string.
+    """
+    for time_format in TIME_FORMAT_LIST:
+        ts, _ = _regex_parse_time(time_string, time_format)
+        if ts is not None:
+            return time_format
+
+
 def _variables_for_parse_time_docstring():
     ret = {}
 
